#include "dlgpreview.h"



dlgPreview::dlgPreview(Widget toplevel, Colormap *cmap, GC *gc, bool isInit, Pane *fir, Pane *cur, bool show, XState *firSt){
  Widget sw, widget;


  gpDisplay  = XtDisplay(toplevel);
  gpGC       = gc;

  if ( !isInit ) {
    firstPane = fir;
    curPane   = cur;
    firstPane->prev = lastPane = fir->prev;
    lastPane->next  = firstPane;
  }
  else
    firstPane = curPane = lastPane = 0;


  selected.red   = 65535;
  selected.green = 65535;
  selected.blue  = 0;
  unselected.red   = 0;
  unselected.green = 0;
  unselected.blue  = 0;
  XAllocColor(gpDisplay, (*cmap), &selected);
  XAllocColor(gpDisplay, (*cmap), &unselected);

  dialog = XtVaCreatePopupShell("PreviewWindow",
				xmDialogShellWidgetClass, toplevel,
				XmNx,    0,         XmNy,     525,
				XmNwidth,           550,
				XmNheight,          145,
				XmNresizePolicy,    XmRESIZE_NONE,
				XmNdeleteResponse,  XmUNMAP,
				NULL);
  XtAddCallback(dialog, XmNpopdownCallback, close_dialog, (XtPointer) dialog);
  XtAddCallback(dialog, XmNdestroyCallback, close_dialog, (XtPointer) dialog);

  pdpane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass, dialog,
			    XmNsashWidth,   1,
			    XmNsashHeight,  1,
			    NULL);

  sw = XtVaCreateManagedWidget("sw", xmScrolledWindowWidgetClass, pdpane,
			       XmNscrollingPolicy,  XmAUTOMATIC,
			       XmNincrement,        10,
			       XmNwidth,            550,
			       XmNheight,           145,
			       NULL);

  previewPane = XtVaCreateManagedWidget("Preview", xmRowColumnWidgetClass, sw,
					XmNorientation,     XmHORIZONTAL,
					XmNleftAttachment,  XmATTACH_FORM,
					XmNtopAttachment,   XmATTACH_FORM,
					XmNspacing,         THUMBNAIL_SPACING,
					NULL);
  
  widget = XtVaCreateManagedWidget("<-", xmPushButtonGadgetClass, previewPane, NULL);
  XtAddCallback(widget, XmNactivateCallback, GoToPrev, NULL);
  
  widget = XtVaCreateManagedWidget("->", xmPushButtonGadgetClass, previewPane, NULL);
  XtAddCallback(widget, XmNactivateCallback, GoToNext, NULL);

  XtVaSetValues(previewPane, XmNbackground, unselected.pixel, NULL);

  if (show)
    {
    XtManageChild(pdpane);
    XtManageChild(dialog);

    if (!isInit)
      RefreshAllPanes(firSt, true, true);
    }

}




dlgPreview::~dlgPreview(){
  XtDestroyWidget(previewPane);
  XtDestroyWidget(dialog);

  previewPane = 0;
  dialog = 0;

}





void dlgPreview::DumpIDs(){

  Pane *pp = firstPane;
  printf("\nID DUMP\n------------\n");
  printf("firstPane=%d\tlastPane=%d\tcurPane=%d\n", firstPane->id, lastPane->id, curPane->id);


  lastPane->next = NULL;
  printf("forward:");
  while (pp)
    {
      printf(" %d -> ", pp->id);
      pp = pp->next;
    }
  lastPane->next = firstPane;


  printf("\nbackward:");
  pp = lastPane;
  firstPane->prev = NULL;
  while (pp)
    {
      printf(" %d -> ", pp->id);
      pp = pp->prev;
    }
  printf("\n");
  firstPane->prev = lastPane;

}





void dlgPreview::NewPane(XState *state, bool showIt){

  if (showIt)
    {
    state->pane = new Pane(previewPane, lastPane==0, state->id, gpGC, firstPane, lastPane, state->lWidth, state->lHeight);
    state->pane->GeneratePreview(state->pix);

    curPane = state->pane;
    if (lastPane){
      lastPane->next  = curPane;
      firstPane->prev = curPane;
      lastPane = curPane;
    }
    else
      lastPane = firstPane = curPane;

    }
  else
    {
    state->pane = new Pane(0, (state->next==state), state->id, NULL, state->next->pane, state->prev->pane, state->lWidth, state->lHeight);
    
    state->next->pane->prev = state->pane;
    state->prev->pane->next = state->pane;

    }

}




void dlgPreview::RefreshAllPanes(XState *xstemp, bool bCreate, bool bGenerate){
  XState *firSt = xstemp;
  Pane *temp    = firSt->pane;


  lastPane->next = NULL;
  while (temp) {
    if (bCreate)
      temp->SetDisplay(previewPane, gpGC, temp==firSt->pane);

    if ((bGenerate) || (bCreate))
      temp->GeneratePreview(xstemp->pix);

    temp->Refresh();
    SetBackground(temp);

    temp   = temp->next;
    xstemp = xstemp->next;

  }
  lastPane->next = firstPane;

}



void dlgPreview::RemovePane(Pane *deleteme, bool isShown, XState *firSt){
  Pane *temp = deleteme->next;

  temp->prev = deleteme->prev;
  temp->prev->next = temp;

  if (isShown) {

    if (deleteme==lastPane)
      curPane = lastPane = temp->prev;
    else if (deleteme==firstPane)
      curPane = firstPane = temp;
    else
      curPane = temp;
  
    firstPane->prev = lastPane;
    lastPane->next  = firstPane;
    
    SelectPane(curPane);
  }

  delete deleteme;

}


void dlgPreview::SelectPane(Pane *target) {
  Pane *temp = firstPane;

  curPane = target;

  lastPane->next = NULL;
  while (temp) {
    SetBackground(temp);
    temp = temp->next;
  }
  lastPane->next = firstPane;

}


void dlgPreview::SetBackground(Pane *target) {
  if (target==curPane)
    XtVaSetValues(target->widget, XmNbackground, selected.pixel, NULL);
  else
    XtVaSetValues(target->widget, XmNbackground, unselected.pixel, NULL);
}
