/*
*  David Eyerman
*  davideyerman@hotmail.com
*  EyeFract 0.5.8
*
*  eyefract.c
*
*/

#define VERSION "0.5.8"
#define EMAIL   "davideyerman@hotmail.com"

#include <X11/cursorfont.h>

#include "eyefract.h"

#ifdef EYECB
#include <Xm/ComboBox.h>
#endif
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/PanedW.h>
#include <Xm/Scale.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <ctype.h>
#include <dirent.h>
#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/timeb.h>

#ifdef ENABLE_JPEG
#include "jpeg/jpeglib.h"
#endif


/* for mouse events */
#define DOWN        0
#define UP          1
#define MOTION      2
#define OUT         3


/*  for jumping around the states  */
#define NEXT           -1
#define PREV           -2
#define ABRT           -3
#define ABRTCLR        -4
#define ABRTPPM        -5
#ifdef ENABLE_JPEG
#define ABRTJPG    -6
#endif


/*  which set we're dealing with  */
#define JULIA       0
#define MANDELBROT  1

#define NUM_COLORS  256

#ifndef SOLARIS
#define THUMBNAIL_SIZE      100
#define THUMBNAIL_PADDING   8
#define THUMBNAIL_SPACING   4
#endif

#define LOAD            2
#define SAVE            3
#ifdef ENABLE_JPEG
#define JPEG        4
#endif

/*  for fractal generation  */
#define CF_NORMAL   1
#define CF_COLOR    2
#define CF_PPM      3
#ifdef ENABLE_JPEG
#define CF_JPEG     4
#endif


/*  state context menu constants  */
#define DO_ZOOMIN        0
#define DO_ZOOMOUT       1
#define DO_RECENTER      2
#define DO_JULIA         3
#define DO_MANDELBROT    4


#define DEFAULT_WIDTH    500//250
#define DEFAULT_HEIGHT   500//250
#define DEFAULT_TYPE     MANDELBROT


/*  whether the coordinates are defined as min/max or center/radius  */
#define CS_MINMAX        1
#define CS_CTRRAD        2
short giCoordDisplayStyle = CS_MINMAX;



struct {
  bool bareMode;
  bool verbose;
} options;





struct complex{
    /*  complext number: a + bi  */
    double a;
    double b;
};


#ifndef SOLARIS
struct pane {
    /*  struct for a preview pane  */
    struct state *stat;
    struct pane *next, *prev;
    Widget widget;
};
struct pane *firstPane, *lastPane, *curPane;
short        iNumPreviews=0;                  /*  number of preview panes */
#endif


/*  for passing info when using state context menus  */
struct {
  double xcent, ycent;
} statevars;


struct state {
/*  a "state" structure containing vital information
 *  for quickly re-producing a picture */
    struct state *next, *prev;
    double       dXmin, dXmax, dYmin, dYmax;  /* Cartesian coordinates */
    long         lMaxIterations;              /* the number of iterations before we "give up"  */
    long         lWidth, lHeight;             /* dimensions of pixmap, drawing area */
    bool         bUseColor;
    XColor       colors[NUM_COLORS];          /* colormap */
    int          type;                        /* JULIA or MANDELBROT */
    short        iFunctionIndex;              /* which function in the array this was generated with */
    struct       complex parameter;           /* the complex parameter */
    Pixmap       pix;                         /* the picture of the state */
#ifndef SOLARIS
    short        pwidth, pheight;             /* dimensions of preview pixmap, preview drawing area */
    Pixmap       preview;                     /* thumbnail version */
#endif
};
struct state *firstState, *lastState, *curState;

#ifndef SOLARIS
bool     isPrevShown = false;
XColor   selected, unselected;
Widget   previewDialog, previewPane, pdpane;
#endif


double   dZoomFactor = 3.0;
int      giFractalType = MANDELBROT;       /*  JULIA or MANDELBROT  */


XColor   gaDisplayColors[NUM_COLORS];
short    curCol = -1, prevCol = -1;        /*  index of current, previously selected colors in colors[]  */
XColor   gHiColor, gLowColor;              /*  used for creating colormaps  */

char     *function_lbls[] = { "f(z) = z^2 + c", "f(z) = z^4 + c", "f(z) = z^4 - z + c",
			  "f(z) = z^5 + c", "f(z) = z^2 + c*z (Julia only)", "f(z) = pi * i * e^z",
			  "f(z) = sin( z )", "f(z) = c * cos( z )" };
short    giNumFuncs = XtNumber(function_lbls);
short    giFunctionIndex = 0;

bool     gbInit = true, gbLoadColormapOK;
bool     gbHasChanged = true, stopped;

Widget topLevel, daCanvas;

Widget paramDialog;
Widget funcList;
Widget lblXmin, lblXmax, lblYmin, lblYmax;
Widget xminBox, xmaxBox, yminBox, ymaxBox;
Widget itersBox, zoomBox;
Widget aBox, bBox, widthBox, heightBox;
Widget keepaspect;

Widget colorDialog;
Widget buttons[NUM_COLORS];
Widget copane, appcolbtn, revcolbtn;
Widget rscale, bscale, gscale;

Widget popupmenu;
Widget typeMenu, messageBar, pane, ccpane;

Widget waitDialog;

Widget aboutDialog, aboutPane;

Widget newColormapDialog, newColormapPane, prevhi, prevlow, numStripes;
Widget rs1, rs2, gs1, gs2, bs1, bs2;




XmString  sComma, sOpenPar, sClosePar;
Colormap  gColormap;
Display   *gpDisplay;
GC        gGC;


Pixmap    old, oldpreview;
Pixel     whitepixel, blackpixel;


FILE      *exportFile;
short     export_aborted=0;
char      *export_filename;


/*  for exporting JPEGs  */
#ifdef ENABLE_JPEG
JSAMPLE   *image_buffer;	/* Points to large array of R,G,B-order data */
struct    jpeg_compress_struct cinfo;
#else
long      *image_buffer;
#endif


void create_aboutDialog();
void create_colorDialog();
void create_previewDialog();//bool override_show);
void create_popupmenu();


void byebye(Widget, XtPointer, XtPointer);

void  changeStateColor(Widget widget, XtPointer client, XtPointer call);
void  CreateFractal(double, double, double, double, short);
void  DeleteState(Widget widget, XtPointer client, XtPointer call);
void  GoToState(Widget, XtPointer, XtPointer);
void  SelectColormapEntry(Widget, XtPointer, XtPointer);
short test_values(long *tempi, long *w, long *h);
void  UpdateParametersDialog();

void StateMenuSelect(Widget widget, XtPointer client, XtPointer call);
void TypeMenuSelect(Widget widget, XtPointer client, XtPointer call);
void ViewMenuSelect(Widget widget, XtPointer client, XtPointer call);
void HelpMenuSelect(Widget widget, XtPointer client, XtPointer call);
void PopupMenuSelect(Widget widget, XtPointer client, XtPointer call);



void redraw(Widget w, XtPointer client, XEvent *event){
  /*
    expose and resize event handling function
    refreshes the drawing area to what's on the current pixmap
  */
  if (curState->pix) {
    XCopyArea(gpDisplay, curState->pix, XtWindow(daCanvas), gGC, 0, 0, curState->lWidth, curState->lHeight, 0, 0);
  }
}


#ifndef SOLARIS
void redrawPreviews(Widget w, XtPointer client, XEvent *event){
    /*  expose and resize event handling function for preview pane window
	refreshes the drawing areas to what's on their associated pixmap  */
    struct pane *tempPane = firstPane;

    lastPane->next = NULL;

    if (previewDialog)
        {
	while (tempPane)
	    {
      	    XCopyArea(gpDisplay, tempPane->stat->preview, XtWindow(tempPane->widget), gGC, 0, 0, tempPane->stat->pwidth, tempPane->stat->pheight, THUMBNAIL_PADDING/2+((THUMBNAIL_SIZE-(tempPane->stat->pwidth))/2), THUMBNAIL_PADDING/2+((THUMBNAIL_SIZE-(tempPane->stat->pheight))/2));
       	    if (tempPane==curPane)
       	        XtVaSetValues(tempPane->widget, XmNbackground, selected.pixel, NULL);
       	    else
       	        XtVaSetValues(tempPane->widget, XmNbackground, unselected.pixel, NULL);
            tempPane = tempPane->next;
	    }
	}

    lastPane->next = firstPane;
}
#endif






//
//
//  Controlling the "Abort" dialog during generation
//
//////////////////////////////////////////////////////////

void stop(Widget dialog, XtPointer client, XtPointer call){
/*  set a flag that the user wants to abort the generation, if we can abort  */

    short itype = (int) client;

    stopped=true;
    if ((curState->next==curState) && (itype==CF_NORMAL)) stopped=false;
}

void TimeoutCursors(Boolean on, Boolean interruptable, int type){

    /*  called at start of fractal generation/color change.
     *  this will present the "working" dialog, and will change the mouse
     *  cursors to their appropriate values.
     */
    XEvent        event;
    XmString      str;
    static short  locked;
    static Cursor cursor;
    XSetWindowAttributes attrs;

    if (on)
        locked++;
    else
        locked--;
    if ((locked>1) || (locked==1 && on==0))
        return;

    stopped = false;
    if (!cursor)
        cursor = XCreateFontCursor(gpDisplay, XC_watch);

    attrs.cursor = on? cursor : None;
    XChangeWindowAttributes(gpDisplay, XtWindow(topLevel), CWCursor, &attrs);
    if (paramDialog)
        XChangeWindowAttributes(gpDisplay, XtWindow(paramDialog), CWCursor, &attrs);
    if (colorDialog)
        XChangeWindowAttributes(gpDisplay, XtWindow(colorDialog), CWCursor, &attrs);
#ifndef SOLARIS
    if (previewDialog)
        XChangeWindowAttributes(gpDisplay, XtWindow(previewDialog), CWCursor, &attrs);
#endif
    XFlush(gpDisplay);
    if (on) {
        waitDialog = XmCreateWorkingDialog(topLevel, "Working", NULL, 0);
	XtUnmanageChild((Widget) XmMessageBoxGetChild(waitDialog, XmDIALOG_OK_BUTTON));
	XtUnmanageChild((Widget) XmMessageBoxGetChild(waitDialog, XmDIALOG_HELP_BUTTON));

	if (interruptable){
	    str = XmStringCreateLocalized("Abort");
	    XtVaSetValues(waitDialog, XmNcancelLabelString, str, NULL);
	    XmStringFree(str);
	    XtAddCallback(waitDialog, XmNcancelCallback, stop, (XtPointer) type);
	}
	else
    	    XtUnmanageChild(XmMessageBoxGetChild(waitDialog, XmDIALOG_CANCEL_BUTTON));

	XtManageChild(waitDialog);
    }
    else {
        while (XCheckMaskEvent(gpDisplay, ButtonPressMask | ButtonReleaseMask |
	    ButtonMotionMask | PointerMotionMask | KeyPressMask | LeaveWindowMask, &event)) { }

	XtDestroyWidget(waitDialog);
    }

}

Boolean CheckForInterrupt(){
    /*  if event happened in working dialog, dispatch it.
     *  if not, ignore it
     */
    XEvent event;

    XFlush(gpDisplay);
    XmUpdateDisplay(topLevel);

    while (XCheckMaskEvent(gpDisplay, ButtonPressMask | ButtonReleaseMask |
	ButtonMotionMask | PointerMotionMask | KeyPressMask | LeaveWindowMask, &event)) {

      if ((event.xany.window = XtWindow(waitDialog) ) )
	XtDispatchEvent(&event);

    }

    return stopped;
}







//
//
//  State functions
//
//
///////////////////////////////////////

#ifndef SOLARIS
void ThumbnailSelect(Widget widget, XtPointer client, XEvent *event){
    /*  called when a user clicks on a state thumbnail  */
    short    button=1;
    Widget   w;
    XButtonPressedEvent *b = (XButtonPressedEvent *) event;

    if (event->xbutton.button == Button1) button = 1;
    if (event->xbutton.button == Button3) button = 2;
    if (event->xbutton.button == Button2) button = 3;

    if ((curPane==(struct pane *) client) && (button==1)) {
        return;
    }
    else if (curPane!=(struct pane *) client) {
        curPane = (struct pane *) client;
        curState = curPane->stat;

        GoToState(NULL, (XtPointer) 1000, NULL);
    }

    if (button==2) {
        popupmenu = XmCreatePopupMenu(previewDialog, "State Popup", NULL, 0);
        XtVaSetValues(popupmenu, XmNautoUnmanage, true, NULL);
            
        w = XtVaCreateManagedWidget("Delete", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,         'D',
                            NULL);
        XtAddCallback(w, XmNactivateCallback, DeleteState, NULL);
        w = XtVaCreateManagedWidget("Change Colors", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'C',
                            NULL);
        XtAddCallback(w, XmNactivateCallback, changeStateColor, NULL);
        XmMenuPosition(popupmenu, b);
        XtManageChild(popupmenu);
    }
}
#endif


void NewState(double xn, double xm, double yn, double ym, long w, long h, bool useColor, long i,
                short type, double a, double b){
    /*  creates a new state  */
    char         t[32]="";
    short        in;
    struct state *temp;
    bool switchback = false;

#ifndef SOLARIS
    struct pane  *temppane;
    temppane = (struct pane *) XtMalloc(sizeof(struct pane));
#endif

    temp = (struct state *) XtMalloc(sizeof(struct state));

    /*  if this is the first state, set all pointers appropriately  */
    if (gbInit){
        gbInit = false;

        lastState = temp;
        firstState = temp;
#ifndef SOLARIS
        firstPane = temppane;
        lastPane = temppane;
#endif
    }

    /*  xmin, xmax, ymin, ymax
     *  for some reason, seg faults appear if i put the XmTextSetString elsewhere... */
    temp->dXmin = xn;
    temp->dXmax = xm;
    temp->dYmin = yn;
    temp->dYmax = ym; 

    if (giCoordDisplayStyle==CS_MINMAX)
        {
	sprintf(t, "%.15f", xn);
	XtVaSetValues(xminBox, XmNvalue, t, NULL);
	sprintf(t, "%.15f", xm);
	XtVaSetValues(xmaxBox, XmNvalue, t, NULL);
	sprintf(t, "%.15f", yn);
	XtVaSetValues(yminBox, XmNvalue, t, NULL);
	sprintf(t, "%.15f", ym);
	XtVaSetValues(ymaxBox, XmNvalue, t, NULL);
        }
    else
        {
	sprintf(t, "%.15f", (xm+xn)/2);
        XtVaSetValues(xminBox, XmNvalue, t, NULL);
	sprintf(t, "%.15f", (xm-xn)/2);
	XtVaSetValues(xmaxBox, XmNvalue, t, NULL);
	sprintf(t, "%.15f", (ym+yn)/2);
	XtVaSetValues(yminBox, XmNvalue, t, NULL);
	sprintf(t, "%.15f", (ym-yn)/2);
	XtVaSetValues(ymaxBox, XmNvalue, t, NULL);
        }

    /*  width, height, depth (color/bw)  */
    temp->lWidth  = w;
    temp->lHeight = h;
    temp->bUseColor = useColor;

    /*  set color menu item  */
    if (useColor)
        XtVaSetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, true, NULL);
    else
        XtVaSetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, false, NULL);

    /*  max iterations, julia/mandel  */
    temp->lMaxIterations = i;
    temp->type   = type;

    /*  which function to iterate  */
    temp->iFunctionIndex = giFunctionIndex;

    /*  important only if type==JULIA  */
    temp->parameter.a = a;
    temp->parameter.b = b;

    /*  links  */
    temp->next = firstState;
    temp->prev = lastState;
    lastState->next = temp;
    firstState->prev = temp;
#ifndef SOLARIS
    temppane->next = firstPane;
    temppane->prev = lastPane;
    lastPane->next = temppane;
    firstPane->prev = temppane;
#endif

    /*  colormap  */
    for (in=0; in<NUM_COLORS; in++){
        temp->colors[in].red   = gaDisplayColors[in].red;
        temp->colors[in].green = gaDisplayColors[in].green;
        temp->colors[in].blue  = gaDisplayColors[in].blue;
        
        XAllocColor(gpDisplay, gColormap, &temp->colors[in]);
    }

    lastState = temp;
    curState = temp;

#ifndef SOLARIS
    /*  set preview width and height, based on actual dimensions  */
    if (w>=h) {
        temp->pwidth  = 100;
        temp->pheight = 100*h/w;
    } else {
        temp->pwidth  = 100*w/h;
        temp->pheight = 100;
    }
    lastPane = temppane;
    curPane = temppane;
    temppane->stat = curState;

    iNumPreviews++;

    if ((temp==firstState) && (temp==lastState))
      {
	if (!options.bareMode)
	  {
	  options.bareMode = true;
	  switchback = true;
	  }


	if (!previewDialog)
	  /*  put this so we don't get problems with the next statements  */
	  create_previewDialog();//0);

	
	if (switchback)
	  options.bareMode = false;
      }


    /*  create and setup associated thumbnail pane  */
    temppane->widget =  XtVaCreateManagedWidget("sep", xmDrawingAreaWidgetClass, previewPane,
		    XmNbackground,       blackpixel,
                    XmNwidth,            THUMBNAIL_SIZE+THUMBNAIL_PADDING,
                    XmNheight,           THUMBNAIL_SIZE+THUMBNAIL_PADDING,
                    XmNresizePolicy,     XmRESIZE_ANY,
                    XmNleftAttachment,   (iNumPreviews>0)? XmATTACH_WIDGET : XmATTACH_FORM,
                    XmNleftWidget,       (iNumPreviews>0)? temppane->prev->widget : NULL,
                    NULL);
    XtAddEventHandler(temppane->widget, ExposureMask, false, (XtEventHandler) redrawPreviews, (XtPointer) NULL);
    XtAddCallback(temppane->widget, XmNresizeCallback, (XtCallbackProc) redrawPreviews, (XtPointer) NULL);
    XtAddEventHandler(temppane->widget, ButtonReleaseMask, false, (XtEventHandler) ThumbnailSelect, (XtPointer) temppane);

    temp->preview = XCreatePixmap(gpDisplay, XtWindow(daCanvas), temp->pwidth, temp->pheight, DefaultDepthOfScreen(XtScreen(daCanvas)) );
    XFillRectangle(gpDisplay, temp->preview, gGC, 0, 0, temp->pwidth, temp->pheight);
#endif


    /*  pixmap  */
    temp->pix = XCreatePixmap(gpDisplay, XtWindow(daCanvas), w, h, DefaultDepthOfScreen(XtScreen(daCanvas)) );

    if (temp->prev == temp) {
        XSetForeground(gpDisplay, gGC, blackpixel);
	XFillRectangle(gpDisplay, temp->pix, gGC, 0, 0, w, h);
    }
    else
      XCopyArea(gpDisplay, temp->prev->pix, temp->pix, gGC, 0, 0, w, h, 0, 0);

}

void DeleteState(Widget widget, XtPointer client, XtPointer call){
    /*  free all memory related to that state, update links in lists  */
#ifndef SOLARIS
    struct pane  *ptemp  = curPane->prev;
#endif
    struct state *pstate = curState->prev;


if (curState->next != curState) {
#ifndef SOLARIS
    iNumPreviews--;
    ptemp->next = curPane->next;
    ptemp->next->prev = ptemp;
#endif
    pstate->next = curState->next;
    pstate->next->prev = pstate;

    if (curState==lastState) {
#ifndef SOLARIS
	lastPane = ptemp;
	firstPane->prev = ptemp;
#endif
	lastState = pstate;
	firstState->prev = pstate;
    }
    else if (curState==firstState) {
#ifndef SOLARIS
	firstPane = ptemp->next;
	firstPane->prev = lastPane;
	lastPane->next = firstPane;
#endif
	firstState = pstate->next;
	firstState->prev = lastState;
	lastState->next = firstState;
    }

#ifndef SOLARIS
    XtUnmanageChild(curPane->widget);
    XtDestroyWidget(curPane->widget);
    XtFree((char *) curPane);
    XFreePixmap(gpDisplay, curState->preview);
    curPane  = ptemp->next;
#endif

    XFreePixmap(gpDisplay, curState->pix);

    XtFree((char *) curState);

    curState = pstate->next;
    GoToState((Widget) NULL, (XtPointer) 1000, (XtPointer) NULL);

    create_popupmenu();
    }
}





void SetMessageBarCoords(double a, double b, double c, double d, short howManyPairs){
    /*  sets the status bar message to the coordinates (a,b) [ -> (c,d) ]  */
    char      buf[24];
    XmString  mess, pair2;

    /*  display (x, y) pair in status bar  */
    XtVaGetValues(messageBar, XmNlabelString, &mess, NULL);
    XtVaGetValues(messageBar, XmNlabelString, &pair2, NULL);
    sprintf(buf, "%.8f", a);
    mess = XmStringConcat(sOpenPar, XmStringCreateLocalized(buf));
    mess = XmStringConcat(mess, sComma);
    sprintf(buf, "%.8f", b);
    mess = XmStringConcat(mess, XmStringCreateLocalized(buf));
    mess = XmStringConcat(mess, sClosePar);

    /*  if 2 pairs of coords given, display them as:
     *  (a, b) -> (c, d)  */
    if (howManyPairs==2){
        pair2 = XmStringConcat(XmStringCreateLocalized(" -> "), sOpenPar);
        sprintf(buf, "%.8f", c);
        pair2 = XmStringConcat(pair2, XmStringCreateLocalized(buf));
        pair2 = XmStringConcat(pair2, sComma);
        sprintf(buf, "%.8f", d);
        pair2 = XmStringConcat(pair2, XmStringCreateLocalized(buf));
        pair2 = XmStringConcat(pair2, sClosePar);
        mess = XmStringConcat(mess, pair2);
    }

    XtVaSetValues(messageBar, XmNlabelString, mess, NULL);

    XmStringFree(mess);
    XmStringFree(pair2);
}








void create_popupmenu(){
    Widget widget;

    if (popupmenu) {
      XtUnmanageChild(popupmenu);
      XtDestroyWidget(popupmenu);
    }
  
#ifdef SOLARIS
    popupmenu = XmCreatePopupMenu(XtParent(daCanvas), "State Popup", NULL, 0);
#else
    popupmenu = XmCreatePopupMenu(daCanvas, "State Popup", NULL, 0);
#endif

    XtVaSetValues(popupmenu, XmNautoUnmanage, true, NULL);

    if (curState->next != curState) {
        widget = XtVaCreateManagedWidget("Delete", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,         'D',
                            NULL);
	XtAddCallback(widget, XmNactivateCallback, DeleteState, NULL);
    }
    widget = XtVaCreateManagedWidget("Change Colors", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'C',
                            NULL);
    XtAddCallback(widget, XmNactivateCallback, changeStateColor, NULL);


    widget = XtVaCreateManagedWidget("Zoom in", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'i',
                            NULL);
    XtAddCallback(widget, XmNactivateCallback, PopupMenuSelect, (XtPointer) DO_ZOOMIN);
    widget = XtVaCreateManagedWidget("Zoom out", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'o',
                            NULL);
    XtAddCallback(widget, XmNactivateCallback, PopupMenuSelect, (XtPointer) DO_ZOOMOUT);
    widget = XtVaCreateManagedWidget("Re-center image", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'R',
                            NULL);
    XtAddCallback(widget, XmNactivateCallback, PopupMenuSelect, (XtPointer) DO_RECENTER);


    if (curState->type==MANDELBROT) {
        widget = XtVaCreateManagedWidget("Create Julia", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'J',
                            NULL);
        XtAddCallback(widget, XmNactivateCallback, PopupMenuSelect, (XtPointer) DO_JULIA);
    }
    /*    else {
        widget = XtVaCreateManagedWidget("Go to Mandel coords", xmPushButtonWidgetClass, popupmenu,
                            XmNmnemonic,   'M',
                            NULL);
        XtAddCallback(widget, XmNactivateCallback, (XtCallbackProc) PopupMenuSelect, DO_MANDELBROT);
	}*/
}



void mouseEvent(Widget w, XtPointer client, XEvent *event){
    /* function for handling all mouse input events */
    static Position  x, y, last_x, last_y;
    short            type=(int) client;
    static short     button=-1;
    static double    tx, ty, ux, uy;
    char             buf[32];
    XButtonPressedEvent  *b;

    if (type == DOWN){
        /* mouse pressed event
         * do nothing, just store coordinates
         * and figure out which button was pressed
         */
        x = event->xbutton.x;
        y = event->xbutton.y;
        last_x = event->xbutton.x;
        last_y = event->xbutton.y;
        if (event->xbutton.button == Button1) button = 1;
        if (event->xbutton.button == Button3) button = 2;
        if (event->xbutton.button == Button2) button = 3;

        /*  convert x and y to cartesian coordinates, for message bar update  */

        tx = curState->dXmin + (x * (curState->dXmax-curState->dXmin)) / curState->lWidth;
        ty = curState->dYmax - (y * (curState->dYmax-curState->dYmin)) / curState->lHeight;

    }
    else if ((type == MOTION) && (button==1)){
        /* mouse motion event from primary mouse button */

        /* draw a rectangle with dashed lines from where the
         * user first clicked to where the mouse pointer is now.
         * this selection will be the area to be zoomed in on once the
         * user stops dragging the mouse and releases the button
         */
        GC         gct;
        XGCValues  gcv;
        bool    isRatio;
        double     true_r, new_r;
        long       h, wd, new_x, new_y;

        new_x=event->xmotion.x;
        new_y=event->xmotion.y;

        XtVaGetValues(keepaspect, XmNset, &isRatio, NULL);
        if (isRatio) {
            true_r = (double) (curState->lWidth)/(curState->lHeight);

            wd = (long) abs(new_x-x);
            h = (long) abs(new_y-y);
            new_r = wd/h;

            if (true_r==new_r){;}
            else {
                if (true_r>new_r){
                    wd = (long) (true_r*h);
                    new_x = (new_x<x)? x-wd : x+wd;
                }
                else{
                    h = (long) (wd/true_r);
                    new_y = (new_y<y)? y-h : y+h;
                }
            }
        }

        /*  update message bar with coordinates of box  */
        ux = curState->dXmin + (new_x * (curState->dXmax-curState->dXmin)) / curState->lWidth;
        uy = curState->dYmax - (new_y * (curState->dYmax-curState->dYmin)) / curState->lHeight;

        SetMessageBarCoords(tx, ty, ux, uy, 2);

        gct = XCreateGC(gpDisplay, XtWindow(w), 0, NULL);

        XSetLineAttributes(gpDisplay, gct, 1, LineOnOffDash, CapRound, JoinRound);

        /*  draw lines over previous lines with last_x, last_y
            draw new ones for xmotion.x and xmotion.y
    
            NOTE: XDrawRectangle is not used here in order to
            use the dotted lines   */
        gcv.foreground = whitepixel;
        XSetForeground(gpDisplay, gct, gcv.foreground);
        XSetFunction(gpDisplay, gct, GXxor);

        XDrawLine(gpDisplay, XtWindow(w), gct, x, y, last_x, y);
        XDrawLine(gpDisplay, XtWindow(w), gct, last_x, y, last_x, last_y);
        XDrawLine(gpDisplay, XtWindow(w), gct, last_x, last_y, x, last_y);
        XDrawLine(gpDisplay, XtWindow(w), gct, x, last_y, x, y);

        XDrawLine(gpDisplay, XtWindow(w), gct, x, y, new_x, y);
        XDrawLine(gpDisplay, XtWindow(w), gct, new_x, y, new_x, new_y);
        XDrawLine(gpDisplay, XtWindow(w), gct, new_x, new_y, x, new_y);
        XDrawLine(gpDisplay, XtWindow(w), gct, x, new_y, x, y);

        last_x = new_x;
        last_y = new_y;

        XFreeGC(gpDisplay, gct);

    } else if (type == MOTION) {
        /*  get x,y of mouse pointer, convert to cartesian  */

        ux = curState->dXmin + (event->xmotion.x * (curState->dXmax-curState->dXmin)) / curState->lWidth;
        uy = curState->dYmax - (event->xmotion.y * (curState->dYmax-curState->dYmin)) / curState->lHeight;

        /*  set message bar to coordinates of current mouse position  */
        SetMessageBarCoords(ux, uy, 0, 0, 1);

    } else if (type == OUT) {
      /*  mouse cursor left the drawing area, clear the message bar  */

        sprintf(buf, "EyeFract %s", VERSION);
        XtVaSetValues(messageBar, XmNlabelString, XmStringCreateLocalized(buf), NULL);

    } else if (type == UP){
        /* mouse button release event */

        /* if the user dragged the primary button, get the
         * coordinates of rectangle's edge and zoom in
         */

        if ((button==1) && (x!=last_x) && (y!=last_y)) {
             long     tempi, h, w, same=-1;
             bool  cbw;

            /* check on number of iterations,
             * width and height typed values */
            if (test_values(&tempi, &w, &h)==1) {

                XtVaGetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, &cbw, NULL);
		if ((curState->lWidth==w) && (curState->lHeight==h)) same=1;
                NewState(tx<ux? tx : ux,    tx<ux? ux : tx,
                          ty<uy? ty : uy,    ty<uy? uy : ty,
                          w,       h,        cbw,
                          tempi,             curState->type,
                          curState->parameter.a,  curState->parameter.b);

                /* zoom in.  create the fractal for the new coordinates  */
                CreateFractal(tx<ux? tx : ux,    tx<ux? ux : tx,
                               ty<uy? ty : uy,    ty<uy? uy : ty,
                               CF_NORMAL * same);
            }
        }
	else if (button==1)
	  create_popupmenu();
        else if (button>1)
	    {
	        /*  popup menu:  delete, change colors, <sep>,
		    zoom in, zoom out, re-center, create Julia  */

	        statevars.xcent = tx;
	        statevars.ycent = ty;

      		b = (XButtonPressedEvent *) event;

                create_popupmenu();

		XmMenuPosition(popupmenu, b);
		XtManageChild(popupmenu);
            }

        /*  reset this to prevent trouble  */
        button = -1;
    }
}


short test_values(long *tempi, long *w, long *h){
    /*  tests the text boxes of number of iterations, colorOffset,
     *  colorSmoothness, width and height to see if we have valid numbers */
    unsigned short  i;
    char   *t="";

    /*  get the number of iterations from the text box
     *  test to see you have a valid (ie >0 integer) number
     */
    XtVaGetValues(itersBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++)
        if (isalpha(t[i])){
            printf("ERROR: The number of iterations must be a number!\n");
            return 0;
        }
    *tempi = atoi(t);

    /*  repeat process for width  */
    XtVaGetValues(widthBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++)
        if (isalpha(t[i])){
            printf("ERROR: The width must be a positive integer!\n");
            return 0;
        }
    *w=atoi(t);

    /*  repeat process for height  */
    XtVaGetValues(heightBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++)
        if (isalpha(t[i])){
            printf("ERROR: The height must be a positive integer!\n");
            return 0;
        }
    *h=atoi(t);

    if (*tempi<=0) { printf("ERROR: The number of iterations must be positive\n"); return 0; }

    /*  returns 1 if every test passed  */
    return 1;
}


void UpdateParametersDialog(){
    /*  fill all data in the parameters dialog box with what's in curState  */
    Widget  widget;
    char    *t="";

#ifdef EYECB
    if ( (widget=XtNameToWidget(funcList, "*List")) != NULL) {
        XmListSelectPos(widget, curState->iFunctionIndex+1, true);
        giFunctionIndex = curState->iFunctionIndex;
    }

    if ( (widget=XtNameToWidget(funcList, "*Text")) != NULL) {
        t = strdup(function_lbls[giFunctionIndex]);
        XtVaSetValues(widget, XmNvalue, t, NULL);
    }
#else
    giFunctionIndex = curState->iFunctionIndex;
    XmListSelectPos(funcList, giFunctionIndex+1, true);
    XtVaGetValues(funcList, XmNtopPosition, &top, XmNvisibleItemCount, &vis, NULL);
    if (giFunctionIndex+1 < top)
        XmListSetPos(funcList, giFunctionIndex+1);
    else if (giFunctionIndex+1 >= top + vis)
        XmListSetBottomPos(funcList, giFunctionIndex+1);
#endif

    XtVaGetValues(xminBox, XmNvalue, &t, NULL);

    /*  complex parameter  */
    sprintf(t, "%.15f", curState->parameter.a);
    XmTextSetString(aBox, t);
    sprintf(t, "%.15f", curState->parameter.b);
    XmTextSetString(bBox, t);

    /*  xmin, xmax, ymin, ymax  */
    if (giCoordDisplayStyle==CS_MINMAX)
        {
	sprintf(t, "%.15f", curState->dXmin);
	XmTextSetString(xminBox, t);
	sprintf(t, "%.15f", curState->dXmax);
	XmTextSetString(xmaxBox, t);
	sprintf(t, "%.15f", curState->dYmin);
	XmTextSetString(yminBox, t);
	sprintf(t, "%.15f", curState->dYmax);
	XmTextSetString(ymaxBox, t);
        }
    else
        {
	sprintf(t, "%.15f", (curState->dXmax + curState->dXmin)/2);
	XmTextSetString(xminBox, t);
	sprintf(t, "%.15f", (curState->dXmax - curState->dXmin)/2);
	XmTextSetString(xmaxBox, t);
	sprintf(t, "%.15f", (curState->dYmax + curState->dYmin)/2);
	XmTextSetString(yminBox, t);
	sprintf(t, "%.15f", (curState->dYmax - curState->dYmin)/2);
	XmTextSetString(ymaxBox, t);
        }

    /*  set menu type  */
    TypeMenuSelect(aBox, (XtPointer) curState->type, 0);

    /*  max iterations, zoom factor  */
    sprintf(t, "%ld", curState->lMaxIterations);
    XmTextSetString(itersBox, t);
    sprintf(t, "%.3f", dZoomFactor);
    XmTextSetString(zoomBox, t);

    /*  width, height  */
    sprintf(t, "%ld", curState->lWidth);
    XmTextSetString(widthBox, t);
    sprintf(t, "%ld", curState->lHeight);
    XmTextSetString(heightBox, t);

    /*  set color type  */
    if (curState->bUseColor)
        XtVaSetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, true, NULL);
    else
        XtVaSetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, false, NULL);

}


long iterate_func(long mi, struct complex *z, double a, double b, long prevn){
    /*  iterate function, return number of steps until bail  */
    long  n=0;
    static struct complex t;

    t.a = z->a;
    t.b = z->b;

    switch (giFunctionIndex){
        case 0:
	  	  if (prevn!=-1)
	  	    {
	    while (n<prevn)
	      {
                z->a = (t.a * t.a) - (t.b * t.b) + a;
                z->b = (2 * t.a * t.b) + b;

		t.a = z->a;
                t.b = z->b;

                n++;
	      }
		    }

            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^2 + c  */
                z->a = (t.a * t.a) - (t.b * t.b) + a;
                z->b = (2 * t.a * t.b) + b;

		t.a = z->a;
                t.b = z->b;

                n++;
	    }
            break;

       case 1:
	 while ( ( ((z->a)*(z->a))+((z->b)*(z->b)) < 4) && (n < mi) ){
                /*  f(z) = z^4 + c  */
                z->a = (t.a * t.a * t.a * t.a) - (6 * t.a * t.a * t.b * t.b) + (t.b * t.b * t.b * t.b) + a;
                z->b = 4 * t.a * t.b * (t.a * t.a - t.b * t.b) + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 2:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^4 - z + c  */
                z->a = (t.a * t.a * t.a * t.a) - (6 * t.a * t.a * t.b * t.b) + (t.b * t.b * t.b * t.b) - t.a + a;
                z->b = 4 * t.a * t.b * (t.a * t.a - t.b * t.b) - t.b + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 3:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^5 + c  */
                z->a = (t.a * t.a * t.a * t.a * t.a) - (10 * t.a * t.a * t.a * t.b * t.b) + (5 * t.a * t.b * t.b * t.b * t.b) + a;
                z->b = (t.b * t.b * t.b * t.b * t.b) - (10 * t.a * t.a * t.b * t.b * t.b) + (5 * t.a * t.a * t.a * t.a * t.b) + b;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 4:
            while ((((t.a*t.a)+(t.b*t.b))<4) && (n<mi)){
                /*  f(z) = z^2 + cz  */
                z->a = (t.a * a) + (t.a * t.a) - (t.b * b) - (t.b * t.b);
                z->b = (t.a * b) + (t.b * a) + (2 * t.a * t.b);

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 5:
            /*  f(z) = pi * i * e^z  */
            while ((t.a<10) && (n<mi)) {
                z->a = -1 * M_PI * exp(t.a) * cos(t.b);
                z->b = M_PI * exp(t.a) * sin(t.b);

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 6:
            /*  f(z) = sin z  */
            while ((t.a<50) && (n<mi)) {
                z->a = sin(t.a)*(exp(t.b)+exp(-1*t.b))/2;
                z->b = cos(t.a)*(exp(t.b)-exp(-1*t.b))/2;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

        case 7:
            /*  f(z) = a * cos z  */
            while ((t.a<50) && (n<mi)) {
                z->a = (cos(t.a)*(exp(t.b)+exp(-1*t.b))*a - sin(t.a)*(exp(-1*t.b)-exp(t.b))*b)/2;
                z->b = (sin(t.a)*(exp(-1*t.b)-exp(t.b))*a + cos(t.a)*(exp(-1*t.b)-exp(t.b))*b)/2;

                t.a = z->a;
                t.b = z->b;

                n++;
            }
            break;

    }

    return n;

    /* pi * i * tan z */

    /* (1 + .1*i)sin z */

    /* e^(z-1) */

    /* (1/e + .1) * e^z */

    /* (.6 + .8i) * sin z */
}


void CreateFractal(double xmin, double xmax, double ymin, double ymax, short abortable){
    /*  the "meat" of the program  */
    long           tempx, tempy, w, h, mi, n=0, cc=0, prevn=-1;
    short          pw, ph, ty;
    short          aborted=0, useDisplay;
    Widget         widget;
    static char    t[32];
    static double  xfa, yfa, ww, hh;
    static Window  win;
    static struct  complex z;
    struct timeb   tb, tb2;
    long sendh, sendw;
    bool d;

    useDisplay = abortable==CF_PPM? 0 : 1;

    if (abortable==(CF_NORMAL * -1))
        {
	abortable *= -1;
        /*  destroy daCanvas and re-create with new dimensions  */
        widget = XtParent(XtParent(daCanvas));
        if (daCanvas)
            {
            if (popupmenu) XtDestroyWidget(popupmenu);
            XtUnmanageChild(daCanvas);
            XtDestroyWidget(daCanvas);
            }
        daCanvas = XtVaCreateManagedWidget("canvas", xmDrawingAreaWidgetClass, widget,
		XmNbackground,       whitepixel,
                XmNwidth,            curState->lWidth,
                XmNheight,           curState->lHeight,
                XmNresizePolicy,     XmRESIZE_ANY,
                NULL);
        XtAddEventHandler(daCanvas, ExposureMask, false, (XtEventHandler) redraw, (XtPointer) NULL);
        XtAddCallback(daCanvas, XmNresizeCallback, (XtCallbackProc) redraw, (XtPointer) NULL);

        create_popupmenu();
        }

    if (useDisplay)
        win = XtWindow(daCanvas);

    /*  make sure we're all in agreement with our mins/maxs
     *  in the case of a zoom in, these MUST be re-set */
    XtVaGetValues(xminBox, XmNvalue, &t, NULL);
    if (giCoordDisplayStyle==CS_MINMAX)
        {
	sprintf(t, "%.15f", curState->dXmin);
	XmTextSetString(xminBox, t);
	sprintf(t, "%.15f", curState->dXmax);
	XmTextSetString(xmaxBox, t);
	sprintf(t, "%.15f", curState->dYmin);
	XmTextSetString(yminBox, t);
	sprintf(t, "%.15f", curState->dYmax);
	XmTextSetString(ymaxBox, t);
        }
    else
        {
        sprintf(t, "%.15f", (curState->dXmax + curState->dXmin)/2);
	XmTextSetString(xminBox, t);
	sprintf(t, "%.15f", (curState->dXmax - curState->dXmin)/2);
	XmTextSetString(xmaxBox, t);
	sprintf(t, "%.15f", (curState->dYmax + curState->dYmin)/2);
	XmTextSetString(yminBox, t);
	sprintf(t, "%.15f", (curState->dYmax - curState->dYmin)/2);
	XmTextSetString(ymaxBox, t);
        }

    w  = curState->lWidth;
    h  = curState->lHeight;
    mi = curState->lMaxIterations;
    d  = curState->bUseColor;
    ty = curState->type;
#ifndef SOLARIS
    pw = curState->pwidth;
    ph = curState->pheight;
#endif

    if ((abortable==CF_NORMAL) && (curState->next==curState))
        TimeoutCursors(true, false, abortable);
    else
        TimeoutCursors(true, true, abortable);


    /* for every pixel on the screen, translate the pixel coordinates
     * to Cartesian coordinates on the scale of [xmin, xmax] and [ymin, ymax].
     * then iterate the function for that  z=(x+yi)  and record n, the number
     * of iterations before we either got over 2 or reached the maximum
     * number of iterations  */

    /*  ratio of coordinate width/height to pixel width/height  */
    xfa = (xmax-xmin)/w;
    yfa = (ymax-ymin)/h;

#ifndef SOLARIS
    /*  ratio of preview pixmap width/height to actual pixmap width/height  */
    ww = (double) pw / w;
    hh = (double) ph / h;
#endif

#ifdef ENABLE_JPEG
    /*  if we're just getting info for exporting a JPEG, allocate the needed memory  *
    if ((abortable == CF_JPEG) || (abortable == CF_PPM))
        image_buffer = XtMalloc(w*h*3*sizeof(unsigned char));
    #else
    if (abortable == CF_PPM)
    image_buffer = XtMalloc(w*h*3*sizeof(long));//unsigned char));*/
#endif

    /*  if we're changing the color of an existing fractal, set old pixmaps  */
    if (abortable==CF_COLOR){
        old = XCreatePixmap(gpDisplay, XtWindow(daCanvas), w, h, DefaultDepthOfScreen(XtScreen(daCanvas)) );
        XCopyArea(gpDisplay, curState->pix, old, gGC, 0, 0, w, h, 0, 0);
#ifndef SOLARIS
        oldpreview = XCreatePixmap(gpDisplay, XtWindow(daCanvas), pw, ph, DefaultDepthOfScreen(XtScreen(daCanvas)) );
        XCopyArea(gpDisplay, curState->preview, oldpreview, gGC, 0, 0, pw, ph, 0, 0);
#endif
    }

ftime(&tb);

    tempy = 0;
    while ( tempy < h ) {
        sprintf(t, "%.0f%% complete", (double) (tempy*100) / h);
//        XtVaSetValues(waitDialog, XtVaTypedArg, XmNmessageString,  XmRString, t, strlen(t)+1, NULL);
        XtVaSetValues(messageBar, XtVaTypedArg, XmNlabelString,  XmRString, t, strlen(t)+1, NULL);

        if (CheckForInterrupt()){
            /*  if we're here, then user wanted to abort generation  */

            switch (abortable) {
                case CF_COLOR:
                    GoToState(NULL, (XtPointer) ABRTCLR, NULL);
                    break;

                case CF_NORMAL:
                    if (curState->next != curState)
                        GoToState(NULL, (XtPointer) ABRT, NULL);
                    break;

                case CF_PPM:
                    GoToState(NULL, (XtPointer) ABRTPPM, NULL);
                    break;

#ifdef ENABLE_JPEG
                case CF_JPEG:
                    GoToState(NULL, (XtPointer) ABRTJPG, NULL);
                    break;
#endif
            }
            aborted=1;
            break;
        }
	tempx = 0;
	while ( tempx < w ) {

            if (ty) {
                curState->parameter.a = xmin + (tempx * xfa);
                curState->parameter.b = ymax - (tempy * yfa);
                z.a = z.b = 0;
            }
            else {
                z.a = xmin + (tempx * xfa);
                z.b = ymax - (tempy * yfa);
            }
            
            n = iterate_func(mi, &z, curState->parameter.a, curState->parameter.b, (prevn==-1)? prevn : prevn/2);
	    //            printf("n=%ld\tl_c[n].red=%d\tl_c[n].green=%d\tl_c[n].blue=%d\tl_c[n].pixel=%ld\n", n, gaDisplayColors[n % NUM_COLORS].red, gaDisplayColors[n % NUM_COLORS].green, gaDisplayColors[n % NUM_COLORS].blue, gaDisplayColors[n % NUM_COLORS].pixel);

            /* plot point now *
             * but if our depth is BW, just set pp to 65535 (=white)
             */

#ifdef ENABLE_JPEG
	      if ((abortable!=CF_JPEG) && (abortable!=CF_PPM)) {
#else
            if (abortable!=CF_PPM) {
#endif

	      if (n!=prevn)
		{
		if (n==mi)
                    /*  we reached max iterations, make pixel black  */
                    XSetForeground(gpDisplay, gGC, blackpixel);
                else
                    if (d)
                        /*  if depth == COLOR, set point appropriately  */
                        XSetForeground(gpDisplay, gGC, gaDisplayColors[n % NUM_COLORS].pixel);
                    else
                        /*  if depth == BW, set point white  */
                        XSetForeground(gpDisplay, gGC, whitepixel);
		}
                /*  draw the point  */
	      //                XDrawPoint(gpDisplay, win, gGC, tempx, tempy);
		                XDrawPoint(gpDisplay, curState->pix, gGC, tempx, tempy);
#ifdef ENABLE_JPEG
            } else if (abortable ==CF_JPEG) {
                /*  set info in image_buffer for JPEG export  */
                if (n==mi)
                    {
                    image_buffer[cc++] = 0;
                    image_buffer[cc++] = 0;
                    image_buffer[cc++] = 0;
                    }
                else
                    {
                    image_buffer[cc++] = curState->colors[n % NUM_COLORS].red   >> 8;
                    image_buffer[cc++] = curState->colors[n % NUM_COLORS].green >> 8;
                    image_buffer[cc++] = curState->colors[n % NUM_COLORS].blue  >> 8;
                    }
#endif
            } else {
                if (n==mi)
                    {
                    image_buffer[cc++] = 0;
                    image_buffer[cc++] = 0;
                    image_buffer[cc++] = 0;
                    }
                else
                    {
                    image_buffer[cc++] = curState->colors[n % NUM_COLORS].red;
                    image_buffer[cc++] = curState->colors[n % NUM_COLORS].green;
                    image_buffer[cc++] = curState->colors[n % NUM_COLORS].blue;
                    }
            }
	prevn = n;
	tempx++;
        } // x-for
    tempy++;
    } //y-for

ftime(&tb2);
printf("time: %ld\n", (tb2.time*1000 + tb2.millitm) - (tb.time*1000 + tb.millitm));

    /*  turn off working dialog, reset cursors  */
    TimeoutCursors(false, false, abortable);



    //
    // HERE!!! make this an option
    //
    ///////////////////////////////


    XCopyArea(gpDisplay, curState->pix, XtWindow(daCanvas), gGC, 0, 0, curState->lWidth, curState->lHeight, 0, 0);

    sprintf(t, "EyeFract %s", VERSION);
    XtVaSetValues(messageBar, XtVaTypedArg, XmNlabelString,  XmRString, t, strlen(t)+1, NULL);

#ifndef SOLARIS
    if (!aborted)
        /*  if we didn't abort, set preview pixmap  */
        for (tempx=0; tempx<pw; tempx++){
           for (tempy=0; tempy<ph; tempy++){
	     sendw = (long) floor(tempx / ww);
	     sendh = (long) floor(tempy / hh);
                XCopyArea(gpDisplay, curState->pix, curState->preview, gGC, sendw, sendh, 1, 1, tempx, tempy);
            }
        }
#endif

    /*  if we are just changing colors, now set state's colors  */
    if (abortable==CF_COLOR)
        for (tempx=0; tempx<NUM_COLORS; tempx++){
            curState->colors[tempx].red   = gaDisplayColors[tempx].red;
            curState->colors[tempx].green = gaDisplayColors[tempx].green;
            curState->colors[tempx].blue  = gaDisplayColors[tempx].blue;

            XAllocColor(gpDisplay, gColormap, &curState->colors[tempx]);
        }


    /*  re-install the handlers  */
    XtAddEventHandler(daCanvas, ButtonPressMask, false, (XtEventHandler) mouseEvent, (XtPointer) DOWN);
    XtAddEventHandler(daCanvas, ButtonReleaseMask, false, (XtEventHandler) mouseEvent, (XtPointer) UP);
    XtAddEventHandler(daCanvas, ButtonMotionMask, false, (XtEventHandler) mouseEvent, (XtPointer) MOTION);
    XtAddEventHandler(daCanvas, PointerMotionMask, false, (XtEventHandler) mouseEvent, (XtPointer) MOTION);
    XtAddEventHandler(daCanvas, LeaveWindowMask, false, (XtEventHandler) mouseEvent, (XtPointer) OUT);
    gbHasChanged = false;

#ifndef SOLARIS
    if (isPrevShown)
        redrawPreviews(NULL, NULL, NULL);
#endif

}


void changeStateColor(Widget widget, XtPointer client, XtPointer call){
    /*  regenerate the state with new colors  */

    CreateFractal(curState->dXmin, curState->dXmax, curState->dYmin, curState->dYmax, CF_COLOR);

#ifndef SOLARIS
    if (isPrevShown)
        redrawPreviews(NULL, NULL, NULL);
#endif

}


void byebye(Widget w, XtPointer client, XtPointer callback){
    /*  run this function on exit.  free all memory, then exit  */
    struct state *temp;
#ifndef SOLARIS
    struct pane  *ptemp;
#endif

    curState = firstState;
    lastState->next = NULL;

#ifndef SOLARIS
    curPane = firstPane;
    lastPane->next = NULL;
#endif

    while (curState) {
        XFreePixmap(gpDisplay, curState->pix);
#ifndef SOLARIS
        XFreePixmap(gpDisplay, curState->preview);
#endif

	//        XFreeColors(gpDisplay, gColormap, curState->colors, NUM_COLORS, 0);

        temp = curState->next;
        XtFree((char *) curState);
        curState = temp;

#ifndef SOLARIS
        ptemp = curPane->next;
        XtFree((char *) curPane);
        curPane = ptemp;
#endif
    }

    XmStringFree(sOpenPar);
    XmStringFree(sClosePar);
    XmStringFree(sComma);
    XFreeColormap(gpDisplay, gColormap);
    XFreeGC(gpDisplay, gGC);
    exit(0);
}


void IterateButtonPressed(Widget wd, XtPointer client, XtPointer callback){
    /*  called when the Iterate button received a click. */
    long     tempi, w, h;
    short    same=-1;
    unsigned short i;
    char     *t="";
    double   txn, txx, tyn, tyx, newa, newb;
    bool  cbw;

    if (!gbHasChanged) return;

    /*  run tests of the number of iterations, color offset, color smoothness,
     *  width and height  */
    if (!test_values(&tempi, &w, &h)) return;

    /* get the typed-in value of Xmin.
     * test to see that it is actually a number */
    XtVaGetValues(xminBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++){
        if (isalpha(t[i])){
	    if (giCoordDisplayStyle==CS_MINMAX)
                printf("ERROR: Xmin must be a number!\n");
	    else
  	        printf("ERROR: X center must be a number!\n");
            return;
        }
    }
    txn=atof(t);

    /*  repeat for Xmax  */
    XtVaGetValues(xmaxBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++){
        if (isalpha(t[i])){
	    if (giCoordDisplayStyle==CS_MINMAX)
                printf("ERROR: Xmax must be a number!\n");
	    else
  	        printf("ERROR: X radius must be a number!\n");
            return;
        }
    }
    txx=atof(t);

    /*  repeat for Ymin  */
    XtVaGetValues(yminBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++){
        if (isalpha(t[i])){
	    if (giCoordDisplayStyle==CS_MINMAX)
                printf("ERROR: Ymin must be a number!\n");
	    else
  	        printf("ERROR: Y center must be a number!\n");
            return;
        }
    }
    tyn=atof(t);

    /*  repeat for Ymax  */
    XtVaGetValues(ymaxBox, XmNvalue, &t, NULL);
    for (i=0; i<strlen(t); i++){
        if (isalpha(t[i])){
	    if (giCoordDisplayStyle==CS_MINMAX)
                printf("ERROR: Ymax must be a number!\n");
	    else
  	        printf("ERROR: Y radius must be a number!\n");
            return;
        }
    }
    tyx=atof(t);

    newa = newb = 0;
    if (giFractalType == JULIA) {
        /*  repeat for a (of a+bi)  */
        XtVaGetValues(aBox, XmNvalue, &t, NULL);
        for (i=0; i<strlen(t); i++){
            if (isalpha(t[i])){
                printf("ERROR: a must be a number!\n");
                return;
            }
        }
        newa=atof(t);

        /*  repeat for b (of a+bi)  */
        XtVaGetValues(bBox, XmNvalue, &t, NULL);
        for (i=0; i<strlen(t); i++){
            if (isalpha(t[i])){
                printf("ERROR: b must be a number!\n");
                return;
            }
        }
        newb=atof(t);

    }

    if (giCoordDisplayStyle==CS_MINMAX)
      {
    /*  some smart tests to run  */
    if (txn>=txx) { printf("ERROR: Xmin must be less than Xmax!\n"); return; }
    if (tyn>=tyx) { printf("ERROR: Ymin must be less than Ymax!\n"); return; }
      }

    XtVaGetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, &cbw, NULL);

    if ((curState->lWidth==w) && (curState->lHeight==h)) same=1;

    if (giCoordDisplayStyle==CS_MINMAX)
      {
      NewState(txn, txx, tyn, tyx, w, h,
		cbw,
		tempi, giFractalType, newa, newb);
    /*  create the fractal  */
    CreateFractal(txn, txx, tyn, tyx, CF_NORMAL * same);
      }
    else
      {
      NewState(txn-txx, txn+txx, tyn-tyx, tyn+tyx, w, h,
		cbw,
		tempi, giFractalType, newa, newb);
    /*  create the fractal  */
    CreateFractal(txn-txx, txn+txx, tyn-tyx, tyn+tyx, CF_NORMAL * same);
      }

}


void load_state(Widget dialog, XtPointer client, XtPointer call){
    /*  load state variables from a file  */
    char    *file = NULL;
    char    buf[32];
    FILE    *fd;
    long    w, h, it;
    unsigned long i;
    short   ok=1, replace=0, same=-1, type;
    double  xi, xa, yi, ya, a=0, b=0;
    XColor  co[NUM_COLORS];
    bool d;
    
    XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) call;
    if (cbs) {
        /* store filename as file  */
        if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &file))
            return;
    }

    if (dialog)
        {
        XtUnmanageChild(dialog);
        XtDestroyWidget(dialog);
        }

    /*  read file  */
    if ((fd = fopen(file, "r")) == NULL) {
        printf("ERROR:  File %s could not be opened\n", file);
        if (curState)
            return;
        exit(1);
    }

    if (file[strlen(file)-3]=='j')
        type = JULIA;
    else
        type = MANDELBROT;

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    xi = atof(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    xa = atof(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    yi = atof(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    ya = atof(buf);

    a = b = 0;
    if (type == JULIA) {
        fscanf(fd, "%s", buf);
        for (i=0; i<strlen(buf); i++){
            if (isalpha(buf[i])){
                ok=0;
                break;
            }
        }
        a = atof(buf);

        fscanf(fd, "%s", buf);
        for (i=0; i<strlen(buf); i++){
            if (isalpha(buf[i])){
                ok=0;
                break;
            }
        }
        b = atof(buf);
    }

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    it = atoi(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    w = atoi(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    h = atoi(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    d = atoi(buf);
    
    i=0;
    while (i<NUM_COLORS) {
        if (fscanf(fd, "%s", buf)==EOF) {
            if (i<NUM_COLORS/2) replace=1;
            break;
        }
        co[i].red = atoi(buf);
        fscanf(fd, "%s", buf);
        co[i].green = atoi(buf);
        fscanf(fd, "%s", buf);
        co[i].blue = atoi(buf);
        i++;
    }

    if (replace)
        for (i=0; i<NUM_COLORS; i++){
            co[i].red   = gaDisplayColors[i].red;
            co[i].green = gaDisplayColors[i].green;
            co[i].blue  = gaDisplayColors[i].blue;
        }

    if (ok) {

        for (i=0; i<NUM_COLORS; i++) {
            gaDisplayColors[i].red   = co[i].red;
            gaDisplayColors[i].green = co[i].green;
            gaDisplayColors[i].blue  = co[i].blue;

            XAllocColor(gpDisplay, gColormap, &gaDisplayColors[i]);

            if (colorDialog)
                XtVaSetValues(buttons[i], XmNbackground, gaDisplayColors[i].pixel, NULL);
        }

	if (curState)
	    if ((curState->lWidth==w) && (curState->lHeight==h)) same=1;

        NewState(xi, xa, yi, ya, w, h, d, it, type, a, b);

        UpdateParametersDialog();

        /*  create the fractal  */
        CreateFractal(xi, xa, yi, ya, CF_NORMAL * same);

        create_popupmenu();
    }
    else
        printf("Error processing file\n");

    fclose(fd);

}


void save_state(Widget dialog, XtPointer client, XtPointer call){
    /*  save state variables to a file  */
    char   *file = NULL;
    FILE   *fd;
    short  i;
    
    XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) call;
    if (cbs) {
        /*  store filename as file  */
        if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &file))
            return;
    }

    /*  write file  */
    if ((fd = fopen(file, "w")) == NULL) {
        printf("ERROR: File could not be opened\n");
        XtUnmanageChild(dialog);
        XtDestroyWidget(dialog);
        return;
    }

    fprintf(fd, "%.15f %.15f %.15f %.15f", curState->dXmin, curState->dXmax, curState->dYmin, curState->dYmax);

    if (curState->type == JULIA)
        fprintf(fd, " %.15f %.15f", curState->parameter.a, curState->parameter.b);

    fprintf(fd, " %ld %ld %ld %d\n", curState->lMaxIterations, curState->lWidth, curState->lHeight, curState->bUseColor);

    for (i=0; i<NUM_COLORS; i++){
        fprintf(fd, "%d %d %d\n", curState->colors[i].red, curState->colors[i].green, curState->colors[i].blue);
    }

    XtUnmanageChild(dialog);
    XtDestroyWidget(dialog);
        
    fclose(fd);
    printf("Save successful - %s\n", file);
    
}


void ResetCoords(Widget dialog, XtPointer client, XtPointer call){
    /*  resets values for xmin, xmax, ymin, ymax  */
    char  buf[32];

    if (giCoordDisplayStyle==CS_MINMAX)
        {
	sprintf(buf, "%.15f", -2.0);
	XtVaSetValues(xminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", 2.0);
	XtVaSetValues(xmaxBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", -2.0);
	XtVaSetValues(yminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", 2.0);
	XtVaSetValues(ymaxBox, XmNvalue, buf, NULL);
        }
    else
        {
        sprintf(buf, "%.15f", 0.0);
	XtVaSetValues(xminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", 2.0);
	XtVaSetValues(xmaxBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", 0.0);
	XtVaSetValues(yminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", 2.0);
	XtVaSetValues(ymaxBox, XmNvalue, buf, NULL);
        }

    TypeMenuSelect(aBox, (XtPointer) giFractalType, 0);
}


void SwitchCoordStyle(Widget w, XtPointer client, XtPointer call){
    char     buf[32]="";
    double   xi, xa, yi, ya;
    double   xc, yc, xr, yr;


    xi = curState->dXmin;
    xa = curState->dXmax;
    yi = curState->dYmin;
    ya = curState->dYmax;
    xc = (xa+xi)/2;
    xr = (xa-xi)/2;
    yc = (ya+yi)/2;
    yr = (ya-yi)/2;

    if (giCoordDisplayStyle==CS_MINMAX)
        {
	giCoordDisplayStyle = CS_CTRRAD;
	XtVaSetValues(w, XmNlabelString, XmStringCreateLocalized("Min./Max."), NULL);

	XtVaSetValues(lblXmin, XmNlabelString, XmStringCreateLocalized("x center:"), NULL);
	XtVaSetValues(lblXmin, XmNheight, 30, NULL);
	XtVaSetValues(lblXmax, XmNlabelString, XmStringCreateLocalized("x radius:"), NULL);
	XtVaSetValues(lblXmax, XmNheight, 30, NULL);

	XtVaSetValues(lblYmin, XmNlabelString, XmStringCreateLocalized("y center:"), NULL);
	XtVaSetValues(lblYmin, XmNheight, 30, NULL);
	XtVaSetValues(lblYmax, XmNlabelString, XmStringCreateLocalized("y radius:"), NULL);
	XtVaSetValues(lblYmax, XmNheight, 30, NULL);

	sprintf(buf, "%.15f", xc);
	XtVaSetValues(xminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", xr);
	XtVaSetValues(xmaxBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", yc);
	XtVaSetValues(yminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", yr);
	XtVaSetValues(ymaxBox, XmNvalue, buf, NULL);
	}
    else
        {
	giCoordDisplayStyle = CS_MINMAX;
	XtVaSetValues(w, XmNlabelString, XmStringCreateLocalized("Cnt./Rad."), NULL);

	XtVaSetValues(lblXmin, XmNlabelString, XmStringCreateLocalized("x min: "), NULL);
	XtVaSetValues(lblXmin, XmNheight, 30, NULL);

	XtVaSetValues(lblXmax, XmNlabelString, XmStringCreateLocalized("x max: "), NULL);
	XtVaSetValues(lblXmax, XmNheight, 30, NULL);

	XtVaSetValues(lblYmin, XmNlabelString, XmStringCreateLocalized("y min: "), NULL);
	XtVaSetValues(lblYmin, XmNheight, 30, NULL);

	XtVaSetValues(lblYmax, XmNlabelString, XmStringCreateLocalized("y max: "), NULL);
	XtVaSetValues(lblYmax, XmNheight, 30, NULL);

	sprintf(buf, "%.15f", xi);
	XtVaSetValues(xminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", xa);
	XtVaSetValues(xmaxBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", yi);
	XtVaSetValues(yminBox, XmNvalue, buf, NULL);
	sprintf(buf, "%.15f", ya);
	XtVaSetValues(ymaxBox, XmNvalue, buf, NULL);
	}
}


void funcSelect(Widget w, XtPointer client, XtPointer call){
    /*  event handler for the function selection combo box  */

#ifdef EYECB
    XmComboBoxCallbackStruct *ccs = (XmComboBoxCallbackStruct *) call;
#else
    XmListCallbackStruct *ccs = (XmListCallbackStruct *) call;
#endif

    if (ccs){
        short i = ccs->item_position;
        if (i-1!=giFunctionIndex) {
            giFunctionIndex = i-1;
            gbHasChanged = true;
        }
    }
    else {
        if (giFunctionIndex==0) return;

        gbHasChanged = true;
        giFunctionIndex = 0;
    }
}


void close_dialog(Widget w, XtPointer client, XtPointer call) {
    XtUnmanageChild((Widget) client);
}


void ParameterEdited(Widget w, XtPointer ca, XtPointer cb){
    /*  now we know we should create a new fractal when "Iterate" is activated  */
    char     *buf="", *b2="";
    bool  KAset;
    long     f, a, b;
    double   dratio, dratioorig, da;

    gbHasChanged = true;


    XtVaGetValues(keepaspect, XmNset, &KAset, NULL);

    if ((w==xmaxBox) && (giCoordDisplayStyle==CS_CTRRAD) && (KAset)) {
            XtRemoveCallback(xmaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtRemoveCallback(ymaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);

            XtVaGetValues(xmaxBox, XmNvalue, &buf, NULL);
            XtVaGetValues(ymaxBox, XmNvalue, &b2, NULL);

	    dratioorig = ((curState->dXmax-curState->dXmin)/2) / ((curState->dYmax-curState->dYmin)/2);
	    da = atof(buf);
            dratio = atof(buf) / atof(b2);

            if (dratio!=dratioorig) {
                sprintf(buf, "%.15f", da/dratioorig);
                XtVaSetValues(ymaxBox, XmNvalue, buf, NULL);
            }

            XtAddCallback(xmaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtAddCallback(ymaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);        
    }
    if ((w==ymaxBox) && (giCoordDisplayStyle==CS_CTRRAD) && (KAset)) {
            XtRemoveCallback(xmaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtRemoveCallback(ymaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);

            XtVaGetValues(xmaxBox, XmNvalue, &buf, NULL);
            XtVaGetValues(ymaxBox, XmNvalue, &b2, NULL);

	    dratioorig = ((curState->dXmax-curState->dXmin)/2) / ((curState->dYmax-curState->dYmin)/2);
	    da = atof(b2);
            dratio = atof(buf) / atof(b2);

            if (dratio!=dratioorig) {
                sprintf(buf, "%.15f", da*dratioorig);
                XtVaSetValues(xmaxBox, XmNvalue, buf, NULL);
            }

            XtAddCallback(xmaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtAddCallback(ymaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);        
    }


    /*  if change was made in width or height boxes, and aspect ratio is set,
     *  set the other one (height or width) to the correct, adjusted value  */
    if ((w==widthBox) && (KAset)) {

            /*  remove the callbacks to prevent crap values, and infinite looping  */
            XtRemoveCallback(widthBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtRemoveCallback(heightBox, XmNvalueChangedCallback, ParameterEdited, NULL);

            XtVaGetValues(widthBox, XmNvalue, &buf, NULL);
            XtVaGetValues(heightBox, XmNvalue, &b2, NULL);
            a = atoi(buf);
            b = atoi(b2);
            f = a * curState->lHeight/curState->lWidth;
            if (f!=b) {
                sprintf(buf, "%ld", f);
                XtVaSetValues(heightBox, XmNvalue, buf, NULL);
            }

            XtAddCallback(widthBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtAddCallback(heightBox, XmNvalueChangedCallback, ParameterEdited, NULL);
    }

    if ((w==heightBox) && (KAset)) {
            /*  remove the callbacks to prevent crap values, and infinite looping  */
            XtRemoveCallback(widthBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtRemoveCallback(heightBox, XmNvalueChangedCallback, ParameterEdited, NULL);

            XtVaGetValues(widthBox, XmNvalue, &buf, NULL);
            XtVaGetValues(heightBox, XmNvalue, &b2, NULL);
            a = atoi(buf);
            b = atoi(b2);
            f = a * curState->lWidth/curState->lHeight;
            if (f!=b) {
                sprintf(buf, "%ld", f);
                XtVaSetValues(widthBox, XmNvalue, buf, NULL);
            }

            XtAddCallback(widthBox, XmNvalueChangedCallback, ParameterEdited, NULL);
            XtAddCallback(heightBox, XmNvalueChangedCallback, ParameterEdited, NULL);
    }
}


void create_paramDialog(){
    Widget  admin, lc, widget, colorbw, form;
    short   i;
    XmStringTable str_list;

    if (!paramDialog) {

        paramDialog = XtVaCreatePopupShell("Parameters", xmDialogShellWidgetClass, topLevel,
            XmNx,    505,       XmNy,     0,
            XmNwidth,           250,
            XmNresizePolicy,    XmRESIZE_NONE,
	    XmNdeleteResponse,  XmUNMAP,
	    NULL);

        pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass, paramDialog,
            XmNsashWidth,   1,
            XmNsashHeight,  1,
            NULL);

        admin  = XtVaCreateManagedWidget("top", xmFormWidgetClass, pane,
                XmNnumColumns,              1,
                XmNorientation,             XmVERTICAL,
                XmNpacking,                 XmPACK_TIGHT,
                XmNisAligned,               true,
                XmNentryVerticalAlignment,  XmALIGNMENT_CONTENTS_TOP,
                XmNleftAttachment,          XmATTACH_FORM,
                XmNbottomAttachment,        XmATTACH_FORM,
                XmNtopAttachment,           XmATTACH_FORM,
                NULL);

        str_list = (XmStringTable) XtMalloc(giNumFuncs * sizeof(XmString));
        for (i=0; i<giNumFuncs; i++)
            str_list[i] = XmStringCreateLocalized(function_lbls[i]);

#ifdef EYECB
        funcList = XtVaCreateManagedWidget("funcCombBox", xmComboBoxWidgetClass, admin,
                XmNcomboBoxType,      XmDROP_DOWN_LIST,
                XmNvisibleItemCount,  giNumFuncs>12? 12 : giNumFuncs,
                XmNitemCount,         giNumFuncs,
                XmNheight,            30,
                XmNselectionPolicy,   XmBROWSE_SELECT,
                NULL);
        XtAddCallback(funcList, XmNselectionCallback, funcSelect, NULL);

        if ( (widget=XtNameToWidget(funcList, "*List")) != NULL ){
            XtVaSetValues(widget,
                        XmNitems,               str_list,
                        XmNvisibleItemCount,    giNumFuncs+1,
                        XmNwidth,               200,
                        XmNitemCount,           giNumFuncs,
                        NULL);
            XmListSelectItem(widget, str_list[0], true);
        }

#else

        XtSetArg(args[0], XmNtopAttachment,       XmATTACH_FORM);
        XtSetArg(args[1], XmNleftAttachment,      XmATTACH_FORM);
        XtSetArg(args[2], XmNrightAttachment,     XmATTACH_FORM);
        XtSetArg(args[3], XmNitems,               str_list);
        XtSetArg(args[4], XmNvisibleItemCount,    3);
        XtSetArg(args[5], XmNitemCount,           giNumFuncs);

        funcList = XmCreateScrolledList(admin, "funcList", args, 6);
        XtManageChild(funcList);

#ifndef SOLARIS
        XmListSelectPos(widget, 0, true);
#endif

        XtAddCallback(funcList, XmNbrowseSelectionCallback, funcSelect, NULL);

        admin  = XtVaCreateManagedWidget("top", xmFormWidgetClass, pane,
                XmNnumColumns,              1,
                XmNorientation,             XmVERTICAL,
                XmNpacking,                 XmPACK_TIGHT,
                XmNisAligned,               true,
                XmNentryVerticalAlignment,  XmALIGNMENT_CONTENTS_TOP,
                XmNleftAttachment,          XmATTACH_FORM,
                XmNbottomAttachment,        XmATTACH_FORM,
                XmNtopAttachment,           XmATTACH_WIDGET,
                XmNtopWidget,               admin,
                NULL);

#endif

        for (i=0; i<giNumFuncs; i++)
            XmStringFree(str_list[i]);
        XtFree((char *) str_list);

        /*  xmin label and textbox  */
        lblXmin  = XtVaCreateManagedWidget("x min: ", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
#ifdef EYECB
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         funcList,
#else
		    XmNtopAttachment,     XmATTACH_FORM,
#endif
                    XmNheight,            30,
                    NULL);
        xminBox = XtVaCreateManagedWidget("xminbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,             "",
                    XmNrightAttachment,   XmATTACH_FORM,
#ifdef EYECB
		    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         funcList,
#else
                    XmNtopAttachment,     XmATTACH_FORM,
#endif
                    XmNleftAttachment,    XmATTACH_WIDGET,
                    XmNleftWidget,        lblXmin,
                    XmNwidth,             135,
                    XmNheight,            30,
		    XmNbackground,        whitepixel,
                    XmNforeground,        blackpixel,
                    NULL);

        /*  xmax label and textbox  */
        lblXmax  = XtVaCreateManagedWidget("x max: ", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         lblXmin,
                    XmNheight,            30,
		    NULL);
        xmaxBox = XtVaCreateManagedWidget("xmaxbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        xminBox,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNleftAttachment,   XmATTACH_WIDGET,
                    XmNleftWidget,       lblXmax,
                    XmNwidth,            135,
                    XmNheight,           30,
		    XmNbackground,       whitepixel,
                    XmNforeground,       blackpixel,
                    NULL);

        /*  ymin label and textbox  */
        lblYmin  = XtVaCreateManagedWidget("y min: ", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         lblXmax,
                    XmNheight,            30,
                    NULL);
        yminBox = XtVaCreateManagedWidget("yminbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        xmaxBox,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNleftAttachment,   XmATTACH_WIDGET,
                    XmNleftWidget,       lblYmin,
                    XmNwidth,            135,
                    XmNheight,           30,
		    XmNbackground,       whitepixel,
                    XmNforeground,       blackpixel,
                    NULL);

        /*  ymax label and textbox  */
        lblYmax  = XtVaCreateManagedWidget("y max: ", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         lblYmin,
                    XmNheight,            30,
                    NULL);
        ymaxBox = XtVaCreateManagedWidget("ymaxbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        yminBox,
                    XmNleftAttachment,   XmATTACH_WIDGET,
                    XmNleftWidget,       lblYmax,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNwidth,            135,
                    XmNheight,           30,
		    XmNbackground,       whitepixel,
                    XmNforeground,       blackpixel,
                    NULL);

        widget = XtVaCreateManagedWidget("Cnt./Rad.", xmPushButtonGadgetClass, admin,
                    XmNtopAttachment,       XmATTACH_WIDGET,
                    XmNtopWidget,           lblYmax,
                    XmNleftAttachment,      XmATTACH_FORM,
                    NULL);
        XtAddCallback(widget, XmNactivateCallback, SwitchCoordStyle, NULL);

        widget = XtVaCreateManagedWidget("Reset Coords", xmPushButtonGadgetClass, admin,
                    XmNtopAttachment,       XmATTACH_WIDGET,
                    XmNtopWidget,           ymaxBox,
                    XmNrightAttachment,     XmATTACH_FORM,
                    NULL);
        XtAddCallback(widget, XmNactivateCallback, ResetCoords, NULL);


        /*  complex parameter labels and textboxes  */
        lc = XtVaCreateManagedWidget("Complex parameter c (a+bi):", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNrightAttachment,   XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         widget,
                    XmNtopOffset,         20,
                    XmNheight,            15,
                    XmNalignment,         XmALIGNMENT_CENTER,
                    NULL);
        aBox = XtVaCreateManagedWidget("complex_a", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        lc,
                    XmNleftAttachment,   XmATTACH_FORM,
                    XmNwidth,            90,
                    XmNheight,           30,
		    XmNforeground,       blackpixel,
		    XmNbackground,       whitepixel,
                    NULL);
        bBox = XtVaCreateManagedWidget("complex_b", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        lc,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNwidth,            90,
                    XmNheight,           30,
		    XmNforeground,       blackpixel,
		    XmNbackground,       whitepixel,
                    NULL);

        if (giFractalType==MANDELBROT) {
            XtVaSetValues(aBox, XtVaTypedArg, XmNbackground, XmRString, "gray", strlen("blck"+1),
                          XmNsensitive, false,
                          NULL);
            XtVaSetValues(bBox, XtVaTypedArg, XmNbackground, XmRString, "gray", strlen("blck"+1),
                          XmNsensitive, false,
                          NULL);
        }

        /*  number of iterations label and textbox  */
        lc = XtVaCreateManagedWidget("Max # of Iterations:", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         aBox,
                    XmNtopOffset,         20,
                    XmNheight,            15,
                    NULL);
        itersBox = XtVaCreateManagedWidget("itersbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        aBox,
                    XmNtopOffset,        20,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNwidth,            65,    
                    XmNheight,           30,
		    XmNforeground,       blackpixel,
		    XmNbackground,       whitepixel,
                    NULL);

        /*  width and height labels and boxes  */
        lc = XtVaCreateManagedWidget("  Width", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         itersBox,
                    XmNtopOffset,         20,
                    XmNheight,            15,
                    NULL);
        widget = XtVaCreateManagedWidget("Height ", xmLabelWidgetClass, admin,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         itersBox,
                    XmNtopOffset,         20,
                    XmNrightAttachment,   XmATTACH_FORM,
                    XmNheight,            15,
                    NULL);
        widget = XtVaCreateManagedWidget("(pixels)", xmLabelWidgetClass, admin,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         itersBox,
                    XmNtopOffset,         20,
                    XmNleftAttachment,    XmATTACH_WIDGET,
                    XmNleftWidget,        lc,
                    XmNrightAttachment,   XmATTACH_WIDGET,
                    XmNrightWidget,       widget,
                    XmNheight,            15,
                    NULL);
        widthBox = XtVaCreateManagedWidget("widthbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        lc,
                    XmNleftAttachment,   XmATTACH_FORM,
                    XmNwidth,            55,    
                    XmNheight,           30,
		    XmNbackground,       whitepixel,
                    XmNforeground,       blackpixel,
                    NULL);
        heightBox = XtVaCreateManagedWidget("heightbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        widget,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNwidth,            55,    
                    XmNheight,           30,
		    XmNforeground,       blackpixel,
		    XmNbackground,       whitepixel,
                    NULL);

        /*  zoom factor label and textbox  */
        lc  = XtVaCreateManagedWidget("Zoom factor ( >1 ): ", xmLabelWidgetClass, admin,
                    XmNleftAttachment,    XmATTACH_FORM,
                    XmNtopAttachment,     XmATTACH_WIDGET,
                    XmNtopWidget,         heightBox,
                    XmNtopOffset,         20,
                    XmNheight,            30,
                    NULL);
        zoomBox = XtVaCreateManagedWidget("zoomBoxbox", xmTextFieldWidgetClass, admin,
                    XmNvalue,            "",
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        heightBox,
                    XmNtopOffset,        20,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNleftAttachment,   XmATTACH_WIDGET,
                    XmNleftWidget,       lc,
                    XmNwidth,            50,
                    XmNheight,           30,
		    XmNforeground,       blackpixel,
                    XmNbackground,       whitepixel,
                    NULL);

        colorbw = XtVaCreateManagedWidget("colorbw1", xmRowColumnWidgetClass, admin,
                    XmNnumColumns,       1,
                    XmNorientation,      XmVERTICAL,
                    XmNentryClass,       xmToggleButtonWidgetClass,
                    XmNpacking,          XmPACK_COLUMN,
                    XmNradioBehavior,    false,
                    XmNtopAttachment,    XmATTACH_WIDGET,
                    XmNtopWidget,        zoomBox,
                    XmNleftAttachment,   XmATTACH_FORM,
                    XmNrightAttachment,  XmATTACH_FORM,
                    XmNheight,           30,
                    NULL);
        keepaspect = XtVaCreateManagedWidget("Maintain aspect ratio", xmToggleButtonWidgetClass, colorbw,
                    XmNtopAttachment,   XmATTACH_FORM,
                    XmNleftAttachment,  XmATTACH_FORM,
                    XmNindicatorType,   XmN_OF_MANY,
                    XmNset,             true,
                    NULL);

        form = XtVaCreateManagedWidget("action_area", xmFormWidgetClass, pane,
                    XmNfractionBase,    5,
                    NULL);

        widget = XtVaCreateManagedWidget("Iterate", xmPushButtonGadgetClass, form,
                    XmNtopAttachment,       XmATTACH_FORM,
                    XmNbottomAttachment,    XmATTACH_FORM,
                    XmNleftAttachment,      XmATTACH_POSITION,
                    XmNleftPosition,        0,
                    XmNrightAttachment,     XmATTACH_POSITION,
                    XmNrightPosition,       2,
                    XmNshowAsDefault,       true,
                    XmNdefaultButtonShadowThickness,    1,
                    NULL);
        XtAddCallback(widget, XmNactivateCallback, IterateButtonPressed, NULL);

        widget = XtVaCreateManagedWidget("Hide", xmPushButtonGadgetClass, form,
                    XmNtopAttachment,       XmATTACH_FORM,
                    XmNbottomAttachment,    XmATTACH_FORM,
                    XmNleftAttachment,      XmATTACH_POSITION,
                    XmNleftPosition,        3,
                    XmNrightAttachment,     XmATTACH_POSITION,
                    XmNrightPosition,       5,
                    XmNdefaultButtonShadowThickness,    1,
                    NULL);
        XtAddCallback(widget, XmNactivateCallback, close_dialog, (XtPointer) paramDialog);

        XtAddCallback(xminBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(xmaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(yminBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(ymaxBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(itersBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(widthBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(heightBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(aBox, XmNvalueChangedCallback, ParameterEdited, NULL);
        XtAddCallback(bBox, XmNvalueChangedCallback, ParameterEdited, NULL);

    }

    if ( !options.bareMode )
        {
        XtManageChild(pane);
        XtManageChild(paramDialog);
        }
}


void ExportPPM(Widget mdialog, XtPointer client, XtPointer call){
    long i;

    XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) call;
    if (cbs) {
        /*  store filename as file  */
        if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &export_filename))
            return;
    }

    export_aborted = 0;

    if (mdialog)
    {
    XtUnmanageChild(mdialog);
    XtDestroyWidget(mdialog);
    }

    if ( ( exportFile = fopen( export_filename, "w" ) ) == NULL ) {
        printf("ERROR:  Unable to open file %s for writing\n", export_filename);
        return;
    }

    CreateFractal(curState->dXmin, curState->dXmax, curState->dYmin, curState->dYmax, CF_PPM);

    fprintf( exportFile, "P6\n%ld %ld\n%d\n", curState->lWidth, curState->lHeight, 65535 );
     
    for( i=0; i<3*curState->lWidth*curState->lHeight; i++ ) {
        fprintf( exportFile, "%c", (char) image_buffer[i]);
    }

    fclose(exportFile);

    if (!export_aborted)
        printf("Export successful - %s\n", export_filename);
    else
        unlink(export_filename);

    XtFree((char *) image_buffer);

}


#ifdef ENABLE_JPEG
void ExportJPEG(Widget mdialog, XtPointer client, XtPointer call){
    /*  export current image to a JPEG  */
    char      *file = NULL;
    int       row_stride;            /* physical row width in buffer */
    JSAMPROW  row_pointer[1];        /* pointer to a single row */
    struct jpeg_error_mgr jerr;
    
    XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) call;
    if (cbs) {
        /*  store filename as file  */
        if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &file))
            return;
    }

    export_aborted=0;

    /*  get rid of file selction dialog */
    XtUnmanageChild(mdialog);
    XtDestroyWidget(mdialog);

    /*  run CreateFractal so that it fills global variable image_buffer  */
    CreateFractal(curState->dXmin, curState->dXmax, curState->dYmin, curState->dYmax, CF_JPEG);

    row_stride = curState->lWidth * 3;      /* JSAMPLEs per row in image_buffer  */

    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_compress(&cinfo);

    /*  open file for writing  */
    if ((exportFile = fopen(file, "wb")) == NULL) {
        fprintf(stderr, "ERROR:  Can't open %s\n", file);
        return;
    }
    jpeg_stdio_dest(&cinfo, exportFile);

    /*  set necessary parameters  */
    cinfo.image_width  = curState->lWidth;
    cinfo.image_height = curState->lHeight;
    cinfo.input_components = 3;
    cinfo.in_color_space = JCS_RGB;
    cinfo.dct_method = JDCT_ISLOW;         /* best algorithm */

    jpeg_set_defaults(&cinfo);

    jpeg_set_quality(&cinfo, 100, TRUE);   /* highest quality setting */

    jpeg_start_compress(&cinfo, TRUE);

    while (cinfo.next_scanline < cinfo.image_height) {
        row_pointer[0] = &image_buffer[cinfo.next_scanline * row_stride];
	jpeg_write_scanlines(&cinfo, row_pointer, 1);
    }

    jpeg_finish_compress(&cinfo);
    fclose(exportFile);

    if (!export_aborted)
        printf("Export successful - %s\n", file);
    else
        unlink(file);

    jpeg_destroy_compress(&cinfo);

    XtFree((char *) image_buffer);
}
#endif


void ImportColormap(Widget mdialog, XtPointer client, XtPointer call){
    /*  import colormap from a file  */
    char    *file = NULL;
    FILE    *fd;
    char    t[12];
    short   k;
    unsigned short i;
    XColor  co[NUM_COLORS];
    XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) call;

    gbLoadColormapOK = 1;

    if (cbs) {
        /* store filename as file  */
        if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &file)) {
            gbLoadColormapOK = 0;
            printf("ERROR:  Could not get filename\n");
            return;
        }
    }

    if (mdialog){
        XtUnmanageChild(mdialog);
        XtDestroyWidget(mdialog);
    }

    /*  read file  */
    if ((fd = fopen(file, "r")) == NULL) {
        if (!gbInit)
            printf("ERROR: Colormap file %s could not be opened\n", file);
        gbLoadColormapOK=0;
        return;
    }

    k=0;
    while ((k<NUM_COLORS) && (gbLoadColormapOK)) {
        fscanf(fd, "%s", t);
        for (i=0; i<strlen(t); i++)
            if (isalpha(t[i])){
                gbLoadColormapOK=0;
                break;
            }
        co[k].red = atoi(t);


        fscanf(fd, "%s", t);
        for (i=0; i<strlen(t); i++)
            if (isalpha(t[i])){
                gbLoadColormapOK=0;
                break;
            }
        co[k].green = atoi(t);

        fscanf(fd, "%s", t);
        for (i=0; i<strlen(t); i++)
            if (isalpha(t[i])){
                gbLoadColormapOK=0;
                break;
            }
        co[k].blue = atoi(t);

        k++;
    }
    fclose(fd);

    if (!gbLoadColormapOK) {
        printf("ERROR: %s is not a valid colormap file\n", file);
        return;
    }

    for (k=0; k<NUM_COLORS; k++){
        gaDisplayColors[k].red = co[k].red;
        gaDisplayColors[k].green = co[k].green;
        gaDisplayColors[k].blue = co[k].blue;

        XAllocColor(gpDisplay, gColormap, &gaDisplayColors[k]);

       if (colorDialog)
            XtVaSetValues(buttons[k], XmNbackground, gaDisplayColors[k].pixel, NULL);
    
    }

    gbHasChanged = true;

    prevCol=-1;

    if (colorDialog) SelectColormapEntry(NULL, 0, NULL);

}


void ExportColormap(Widget mdialog, XtPointer client, XtPointer call){
    /*  export colormap to a file  */
    char   *file = NULL;
    FILE   *fd;
    short  i;
    
    XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *) call;
    if (cbs) {
        /*  store filename as file  */
        if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &file))
            return;
    }

    /*  write file  */
    if ((fd = fopen(file, "w")) == NULL) {
        printf("ERROR: File could not be opened\n");
        XtUnmanageChild(mdialog);
        XtDestroyWidget(mdialog);
        return;
    }

    for (i=0; i<NUM_COLORS; i++)
        fprintf(fd, "%d %d %d\n", gaDisplayColors[i].red, gaDisplayColors[i].green, gaDisplayColors[i].blue);

    XtUnmanageChild(mdialog);
    XtDestroyWidget(mdialog);
        
    fclose(fd);
    printf("Save successful - %s\n", file);
    
}



//
//
//   Menu Callbacks
//
//
////////////////////////////////////////////////

void StateMenuSelect(Widget w, XtPointer client, XtPointer call){
    /*  callback function for the State menu  */
    short     item = (int) client;
    Arg       args[1];
    Widget    dialog;
    XmString  mask;

    switch (item) {
        case 0: /*  load file  */
            /*  first set directory filter  */
            mask = XmStringCreateLocalized("states/*.[mj]sn");

            XtSetArg(args[0], XmNdirMask, mask);

            dialog = XmCreateFileSelectionDialog(topLevel, "Load State", args, 1);
            XtAddCallback(dialog, XmNokCallback, load_state, NULL);
            XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) XtUnmanageChild, dialog);

            XmStringFree(mask);
            XtManageChild(dialog);
            break;


        case 1:    /*  save file  */
            if (gbInit) break;

            /*  first set directory filter  */
            if (giFractalType == JULIA)
                mask = XmStringCreateLocalized("states/*.jsn");
            else
                mask = XmStringCreateLocalized("states/*.msn");

            XtSetArg(args[0], XmNdirMask, mask);

            dialog = XmCreateFileSelectionDialog(topLevel, "Save State", args, 1);
            XtAddCallback(dialog, XmNokCallback, save_state, NULL);
            XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) XtUnmanageChild, NULL);

            XmStringFree(mask);

            XtManageChild(dialog);
            break;


        case 2:  /*  Load colormap...  */
            mask = XmStringCreateLocalized("maps/*.map");
            XtSetArg(args[0], XmNdirMask, mask);

            dialog = XmCreateFileSelectionDialog(topLevel, "Import Colormap", args, 1);

            XtAddCallback(dialog, XmNokCallback, ImportColormap, NULL);
            XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) XtUnmanageChild, NULL);

            XmStringFree(mask);

            XtManageChild(dialog);
            break;

        case 3:  /*  Save settings...  */
            mask = XmStringCreateLocalized("maps/*.map");
            XtSetArg(args[0], XmNdirMask, mask);

            dialog = XmCreateFileSelectionDialog(topLevel, "Export Colormap", args, 1);

            XtAddCallback(dialog, XmNokCallback, ExportColormap, NULL);
            XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) XtUnmanageChild, NULL);

            XmStringFree(mask);

            XtManageChild(dialog);
            break;

        case 4:

#ifdef ENABLE_JPEG
        /*  Export JPEG...  */
            mask = XmStringCreateLocalized("images/*.jpg");
            XtSetArg(args[0], XmNdirMask, mask);

            dialog = XmCreateFileSelectionDialog(topLevel, "Export JPEG", args, 1);
            XtAddCallback(dialog, XmNokCallback, ExportJPEG, NULL);
            XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) XtUnmanageChild, NULL);

            XmStringFree(mask);

            XtManageChild(dialog);
            break;

        case 5:
#endif
        /*  Export PPM...  */
            mask = XmStringCreateLocalized("images/*.ppm");
            XtSetArg(args[0], XmNdirMask, mask);

            dialog = XmCreateFileSelectionDialog(topLevel, "Export PPM", args, 1);
            XtAddCallback(dialog, XmNokCallback, ExportPPM, NULL);
            XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) XtUnmanageChild, NULL);

            XmStringFree(mask);

            XtManageChild(dialog);
            break;

#ifdef ENABLE_JPEG
        case 6:
#else
        case 5:
#endif
        /*  quit  */
            byebye(NULL, NULL, NULL);
            break;
    }
}


void TypeMenuSelect(Widget w, XtPointer client, XtPointer call){
    /*  callback function for the Type menu  */
    short   item = (int) client;
    Widget  widget;

    switch (item) {
        case 0: /*  Julia  */
            /* enable the a+bi boxes, set background to white
             * make sure menu radio boxes are set properly  */
            if (giFractalType==JULIA) return;
            giFractalType = JULIA;

            if (aBox){
                XtVaSetValues(aBox, XmNsensitive, true, XmNbackground, whitepixel, NULL);
                XtVaSetValues(bBox, XmNsensitive, true, XmNbackground, whitepixel, NULL);
            }

            if ( (widget = XtNameToWidget(typeMenu, "button_0")) != NULL )
                XtVaSetValues(widget, XmNset, true, NULL);
            if ( (widget = XtNameToWidget(typeMenu, "button_1")) != NULL)
                XtVaSetValues(widget, XmNset, false, NULL);

            gbHasChanged = true;
            break;

        case 1: /*  Mandelbrot  */
            /* disable the a+bi boxes, set background to gray
             * make sure menu radio boxes are set properly  */
            if (giFractalType==MANDELBROT) return;
            giFractalType = MANDELBROT;

            if (aBox){
                XtVaSetValues(aBox, XmNsensitive, false,
                    XtVaTypedArg, XmNbackground, XmRString, "gray", strlen("gray"+1),
                    NULL);
                XtVaSetValues(bBox, XmNsensitive, false,
                    XtVaTypedArg, XmNbackground, XmRString, "gray", strlen("gray"+1),
                    NULL);
            }

            if ( (widget = XtNameToWidget(typeMenu, "button_0")) != NULL )
                XtVaSetValues(widget, XmNset, false, NULL);
	    if ( (widget = XtNameToWidget(typeMenu, "button_1")) != NULL )
                XtVaSetValues(widget, XmNset, true, NULL);

            gbHasChanged = true;
            break;

        case 2: /*  Color  */
            gbHasChanged = true;
            break;
    }
}


void ViewMenuSelect(Widget w, XtPointer client, XtPointer call){
    short item = (int) client;

    switch (item){
        case 0:  /*  parameters  */
	  options.bareMode = false;//gbShow = 1;
            create_paramDialog();
            break;
        case 1:  /*  color  */
            create_colorDialog();
            break;
#ifndef SOLARIS
        case 2:  /*  preview  */
	  options.bareMode = false;//gbShow = 1;
	  create_previewDialog();//gbShow);
            break;
#endif
    }
}


void HelpMenuSelect(Widget w, XtPointer client, XtPointer call){
    short item = (int) client;

    switch (item){
        case 0:  /*  About...  */
            if (!aboutDialog) {
                create_aboutDialog();
            }
            XtManageChild(aboutPane);
            XtManageChild(aboutDialog);

            break;
    }
}


void PopupMenuSelect(Widget widget, XtPointer client, XtPointer call){
    short    which=(int) client, same=-1;
    double   cw, ch, zf, tx, ty;
    char     *t="";
    long     tempi, h, w;
    unsigned short    i;
    bool  cbw;

    switch (which){
        case DO_MANDELBROT:
	  /*  coming soon... maybe...
	      need to determine the x/y radii!!  */
	  break;

        case DO_JULIA:
            NewState(curState->dXmin, curState->dXmax, curState->dYmin, curState->dYmax,
	              curState->lWidth, curState->lHeight, curState->bUseColor, curState->lMaxIterations,
		      JULIA, statevars.xcent, statevars.ycent);

	    UpdateParametersDialog();
	    TypeMenuSelect(NULL, JULIA, NULL);
	    CreateFractal(curState->dXmin, curState->dXmax, curState->dYmin, curState->dYmax, CF_NORMAL);

	    break;

        case DO_ZOOMIN:

	    tx = statevars.xcent;
	    ty = statevars.ycent;


            /*  check on number of iterations,
             *  width and height typed-in values  */
            if (!test_values(&tempi, &w, &h)) return;

            XtVaGetValues(zoomBox, XmNvalue, &t, NULL);
            for (i=0; i<strlen(t); i++)
                if (isalpha(t[i])){
                    printf("ERROR: The zoom factor must be a number!\n");
                    return;
                   }
            zf = atof(t);

            /*  if bad, print an error and return */
            if (zf<1) {
                printf("ERROR: The zoom factor must be greater than 1.\n");
                return;
            }

            /*  we're good to go on the zoom in  */

            /*  get width, height (in cart. coords) and divide them by the zoom factor  */
            dZoomFactor = zf;
            cw = (curState->dXmax - curState->dXmin) / dZoomFactor;
            ch = (curState->dYmax - curState->dYmin) / dZoomFactor;
            XtVaGetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, &cbw, NULL);

	    if ((curState->lWidth==w) && (curState->lHeight==h)) same=1;

            NewState(tx - (cw / 2),    tx + (cw / 2),
                      ty - (ch / 2),    ty + (ch / 2),
                      w,       h,        cbw,
                      tempi,             curState->type,
                      curState->parameter.a,  curState->parameter.b);

            /* zoom in.  create the fractal for the new coordinates  */
            CreateFractal(tx - (cw / 2),    tx + (cw / 2),
                           ty - (ch / 2),    ty + (ch / 2),
                           CF_NORMAL * same);

	    break;

        case DO_ZOOMOUT:
	    tx = statevars.xcent;
	    ty = statevars.ycent;


            /*  check on number of iterations,
             *  width and height typed-in values  */
            if (!test_values(&tempi, &w, &h)) return;

            XtVaGetValues(zoomBox, XmNvalue, &t, NULL);
            for (i=0; i<strlen(t); i++)
                if (isalpha(t[i])){
                    printf("ERROR: The zoom factor must be a number!\n");
                    return;
                   }
            zf = atof(t);

            /*  if bad, print an error and return */
            if (zf<1) {
                printf("ERROR: The zoom factor must be greater than 1.\n");
                return;
            }

            /*  we're good to go on the zoom out  */

            /*  get width, height (in cart. coords) and divide them by the zoom factor  */
            dZoomFactor = zf;
            cw = (curState->dXmax - curState->dXmin) * dZoomFactor;
            ch = (curState->dYmax - curState->dYmin) * dZoomFactor;
            XtVaGetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, &cbw, NULL);

	    if ((curState->lWidth==w) && (curState->lHeight==h)) same=1;

            NewState(tx - (cw / 2),    tx + (cw / 2),
                      ty - (ch / 2),    ty + (ch / 2),
                      w,       h,        cbw,
                      tempi,             curState->type,
                      curState->parameter.a,  curState->parameter.b);

            /* zoom in.  create the fractal for the new coordinates  */
            CreateFractal(tx - (cw / 2),    tx + (cw / 2),
                           ty - (ch / 2),    ty + (ch / 2),
                           CF_NORMAL * same);
	    break;

        case DO_RECENTER:
            tx = statevars.xcent;
	    ty = statevars.ycent;

	    cw = curState->dXmax - curState->dXmin;
	    ch = curState->dYmax - curState->dYmin;

            if (!test_values(&tempi, &w, &h)) return;

	    if ((curState->lWidth==w) && (curState->lHeight==h)) same=1;

            NewState(tx - (cw / 2),    tx + (cw / 2),
                      ty - (ch / 2),    ty + (ch / 2),
                      w,       h,        cbw,
                      tempi,             curState->type,
                      curState->parameter.a,  curState->parameter.b);

            /* zoom in.  create the fractal for the new coordinates  */
            CreateFractal(tx - (cw / 2),    tx + (cw / 2),
                           ty - (ch / 2),    ty + (ch / 2),
                           CF_NORMAL * same);

	    break;
    }

}










void create_aboutDialog(){
    Widget  lbl, tform, aform;
    char    t[50]="";

    aboutDialog = XtVaCreatePopupShell("About EyeFract", xmDialogShellWidgetClass, topLevel,
                     XmNx,    100,       XmNy,     100,
                     XmNwidth,           200,
                     XmNheight,          200,
                     XmNresizePolicy,    XmRESIZE_NONE,
                     XmNdeleteResponse,  XmDESTROY,
                     NULL);

    aboutPane = XtVaCreateWidget("aboutPane", xmPanedWindowWidgetClass, aboutDialog,
                     XmNsashWidth,   1,
                     XmNsashHeight,  1,
                     NULL);

    tform  = XtVaCreateManagedWidget("top", xmFormWidgetClass, aboutPane,
                     XmNnumColumns,          3,
                     XmNorientation,         XmHORIZONTAL,
                     XmNpacking,             XmPACK_COLUMN,
                     XmNleftAttachment,      XmATTACH_FORM,
                     XmNbottomAttachment,    XmATTACH_FORM,
                     XmNtopAttachment,       XmATTACH_FORM,
                     NULL);

    sprintf(t, "EyeFract %s", VERSION);
    lbl = XtVaCreateManagedWidget(t, xmLabelWidgetClass, tform, 
                     XmNtopAttachment,  XmATTACH_FORM,
                     NULL);
    sprintf(t, "David Eyerman");
    lbl = XtVaCreateManagedWidget(t, xmLabelWidgetClass, tform,
                     XmNtopAttachment,  XmATTACH_WIDGET,
                     XmNtopWidget,      lbl,
                     XmNtopOffset,      8,
                     NULL);
    sprintf(t, "%s", EMAIL);
    lbl = XtVaCreateManagedWidget(t, xmLabelWidgetClass, tform,
                     XmNtopAttachment,  XmATTACH_WIDGET,
                     XmNtopWidget,      lbl,
                     NULL);
    sprintf(t, "http://jhunix.hcf.jhu.edu/~dee1/eyefract.html");
    lbl = XtVaCreateManagedWidget(t, xmLabelWidgetClass, tform,
                     XmNtopAttachment,  XmATTACH_WIDGET,
                     XmNtopWidget,      lbl,
                     NULL);

    aform = XtVaCreateManagedWidget("action_area", xmFormWidgetClass, aboutPane,
                     XmNfractionBase,    5,
                     NULL);

    lbl = XtVaCreateManagedWidget("OK", xmPushButtonGadgetClass, aform,
                     XmNleftAttachment,      XmATTACH_POSITION,
                     XmNleftPosition,        1,
                     XmNrightAttachment,     XmATTACH_POSITION,
                     XmNrightPosition,       4,
                     XmNshowAsDefault,       true,
                     XmNdefaultButtonShadowThickness,    1,
                     NULL);
    XtAddCallback(lbl, XmNactivateCallback, close_dialog, aboutDialog);
}


void UpdateColormapEntryColor(Widget w, XtPointer client, XtPointer call){
    /*  slider was moved
     *  adjust value of the appropriate cell in gaDisplayColors[]
     *  update display  */
    short rgb = (int) client;
    XmScaleCallbackStruct *cbs = (XmScaleCallbackStruct *) call;

    if (curCol<0) return;

    switch (rgb) {
        case DoRed:
            gaDisplayColors[curCol].red = (cbs->value);
            break;
        case DoGreen:
            gaDisplayColors[curCol].green = (cbs->value);
            break;
        case DoBlue:
            gaDisplayColors[curCol].blue = (cbs->value);
            break;
    }

    XAllocColor(gpDisplay, gColormap, &gaDisplayColors[curCol]);
    XtVaSetValues(buttons[curCol], XmNbackground, gaDisplayColors[curCol].pixel, NULL);

    /*  value has changed, allow revert and apply buttons to be sensitive  */
    XtVaSetValues(revcolbtn, XmNsensitive, true, NULL);
    XtVaSetValues(appcolbtn, XmNsensitive, true, NULL);
}


void SelectColormapEntry(Widget w, XtPointer client, XtPointer call){
    /*  set the colors and scales to the selected color in the map
     *  if there was a previously selected color, get rid of the frame around it  */
    Window     win;
    GC         gct;
    XGCValues  gcv;

    if (prevCol>=0) {
        /*  there was a previously-selected color
         *  go over "highlight" lines, set background back to its true color  */
        win = XtWindow(buttons[curCol]);
        gct = XCreateGC(gpDisplay, win, 0, NULL);

        gcv.foreground = whitepixel;
        gcv.line_width = 3;
        XChangeGC(gpDisplay, gct, GCForeground | GCLineWidth, &gcv);
        XSetFunction(gpDisplay, gct, GXxor);
        XDrawLine(gpDisplay, win, gct, 0,  0,  0,  20);
        XDrawLine(gpDisplay, win, gct, 0,  20, 20, 20);
        XDrawLine(gpDisplay, win, gct, 20, 20, 20, 0);
        XDrawLine(gpDisplay, win, gct, 20, 0,  0,  0);
        XFreeGC(gpDisplay, gct);

        XtVaSetValues(buttons[curCol], XmNbackground, curState->colors[curCol].pixel, NULL);
    }

    /*  update prevCol and curCol  */
    if (prevCol==-1)
        prevCol = (int) client;
    else
        prevCol = curCol;
    curCol  = (int) client;

    /*  draw lines around selected color, so that user knows which box was selected  */
    gct = XCreateGC(gpDisplay, XtWindow(buttons[curCol]), 0, NULL);

    gcv.foreground = whitepixel;
    gcv.line_width = 3;
    XChangeGC(gpDisplay, gct, GCForeground | GCLineWidth, &gcv);
    XSetFunction(gpDisplay, gct, GXxor);

    XDrawLine(gpDisplay, XtWindow(buttons[curCol]), gct, 0,  0,  0,  20);
    XDrawLine(gpDisplay, XtWindow(buttons[curCol]), gct, 0,  20, 20, 20);
    XDrawLine(gpDisplay, XtWindow(buttons[curCol]), gct, 20, 20, 20, 0);
    XDrawLine(gpDisplay, XtWindow(buttons[curCol]), gct, 20, 0,  0,  0);
    XFreeGC(gpDisplay, gct);

    /*  now do what function title says, and set the scales  */
    XtVaSetValues(rscale, XmNvalue, gaDisplayColors[curCol].red, NULL);
    XtVaSetValues(gscale, XmNvalue, gaDisplayColors[curCol].green, NULL);
    XtVaSetValues(bscale, XmNvalue, gaDisplayColors[curCol].blue, NULL);

    XtVaSetValues(appcolbtn, XmNsensitive, false, NULL);
    XtVaSetValues(revcolbtn, XmNsensitive, false, NULL);
}


void RevertColormapEntry(Widget w, XtPointer client, XtPointer call){
    /*  eliminate the change to the color in the map  */

    XtVaSetValues(buttons[curCol], XmNbackground, curState->colors[curCol].pixel, NULL);

    gaDisplayColors[curCol].red   = curState->colors[curCol].red;
    gaDisplayColors[curCol].green = curState->colors[curCol].green;
    gaDisplayColors[curCol].blue  = curState->colors[curCol].blue;

    XAllocColor(gpDisplay, gColormap, &gaDisplayColors[curCol]);

    XtVaSetValues(revcolbtn, XmNsensitive, false, NULL);
    XtVaSetValues(appcolbtn, XmNsensitive, false, NULL);
    prevCol = -1;
}


void SaveColormapEntry(Widget w, XtPointer client, XtPointer call){
    /*  accept the change to the color in the map  */
    curState->colors[curCol].red   = gaDisplayColors[curCol].red;
    curState->colors[curCol].green = gaDisplayColors[curCol].green;
    curState->colors[curCol].blue  = gaDisplayColors[curCol].blue;
    XAllocColor(gpDisplay, gColormap, &curState->colors[curCol]);

    XtVaSetValues(revcolbtn, XmNsensitive, false, NULL);
    XtVaSetValues(appcolbtn, XmNsensitive, false, NULL);

    prevCol = -1;
    gbHasChanged = true;
}


void UpdateNewColorPreview(Widget w, XtPointer client, XtPointer call){
    /*  update preview color in create CMAP window  */
    short which = (int) client;
    XmScaleCallbackStruct *cbs = (XmScaleCallbackStruct *) call;

    if (which<0)  { /* color 1 */
        which *= -1;

        switch (which) {
            case DoRed:
                gHiColor.red = (cbs->value);
                break;
            case DoGreen:
                gHiColor.green = (cbs->value);
                break;
            case DoBlue:
                gHiColor.blue = (cbs->value);
                break;
        }
        XAllocColor(gpDisplay, gColormap, &gHiColor);
        XtVaSetValues(prevhi, XmNbackground, gHiColor.pixel, NULL);

    } else {  /* color 2 */

        switch (which) {
            case DoRed:
                gLowColor.red = (cbs->value);
                break;
            case DoGreen:
                gLowColor.green = (cbs->value);
                break;
            case DoBlue:
                gLowColor.blue = (cbs->value);
                break;
        }
        XAllocColor(gpDisplay, gColormap, &gLowColor);
        XtVaSetValues(prevlow, XmNbackground, gLowColor.pixel, NULL);
    }
}


void GenerateColormap(Widget w, XtPointer cli, XtPointer ca){
    /*  generate the colormap, based on XColors gHiColor and gLowColor  */
    short  redch, bluech, greench, str, j, length, remainder=0, in;
    char   *s="";
    unsigned short i;

    XtVaGetValues(numStripes, XmNvalue, &s, NULL);
    for (i=0; i<strlen(s); i++)
        if (isalpha(s[i])){
            printf("ERROR: Invalid format: number of stripes\n");
            return;
        }
    str = atoi(s);

    if ((str<0) || (str>64)) { 
        printf("ERROR: The number of stripes must be in the range of [0, 64]\n");
        return;
    }

    if (str==0) {
        length = NUM_COLORS*2;
        remainder = 0;
    }
    else {
        length = NUM_COLORS/str;
        if (length * str != NUM_COLORS) remainder = NUM_COLORS - (length * str);
    }
    
    redch   = (gHiColor.red   - gLowColor.red)   / (length/2);
    greench = (gHiColor.green - gLowColor.green) / (length/2);
    bluech  = (gHiColor.blue  - gLowColor.blue)  / (length/2);

    for (i=0; i<NUM_COLORS; i++){

        j = i % length;
        if (remainder) {
            for (in=1; in<(str*2); in+=2) {
                if (i==(NUM_COLORS*in)/(str*2)) {
                    gaDisplayColors[i].red   = gaDisplayColors[i - 1].red;
                    gaDisplayColors[i].green = gaDisplayColors[i - 1].green;
                    gaDisplayColors[i].blue  = gaDisplayColors[i - 1].blue;
                    XAllocColor(gpDisplay, gColormap, &gaDisplayColors[i]);
                            
                    XtVaSetValues(buttons[i], XmNbackground, gaDisplayColors[i].pixel, NULL);
                    i++;
                    break;
                }
            }
        }
        if (j<(length/2)) {
            gaDisplayColors[i].red   = gHiColor.red   - ((j+1)*redch);
            gaDisplayColors[i].green = gHiColor.green - ((j+1)*greench);
            gaDisplayColors[i].blue  = gHiColor.blue  - ((j+1)*bluech);
        } else if (j>(length/2)) {
            gaDisplayColors[i].red   = gaDisplayColors[length-1-j].red;
            gaDisplayColors[i].green = gaDisplayColors[length-1-j].green;
            gaDisplayColors[i].blue  = gaDisplayColors[length-1-j].blue;

        } else {
            gaDisplayColors[i].red   = gLowColor.red;
            gaDisplayColors[i].green = gLowColor.green;
            gaDisplayColors[i].blue  = gLowColor.blue;
        }

        XAllocColor(gpDisplay, gColormap, &gaDisplayColors[i]);
        XtVaSetValues(buttons[i], XmNbackground, gaDisplayColors[i].pixel, NULL);

    } /* end for each cell */

    gbHasChanged = true;
    prevCol = -1;

    XtVaSetValues(rscale, XmNvalue, gaDisplayColors[curCol].red, NULL);
    XtVaSetValues(gscale, XmNvalue, gaDisplayColors[curCol].green, NULL);
    XtVaSetValues(bscale, XmNvalue, gaDisplayColors[curCol].blue, NULL);

    XtUnmanageChild(newColormapDialog);

}


void MapListSelection(Widget w, XtPointer cli, XtPointer ca){
  XmListCallbackStruct *cbs = (XmListCallbackStruct *) ca;
  char *choice, base[270]="./maps/";
  XmFileSelectionBoxCallbackStruct f;

  XmStringGetLtoR(cbs->item, XmFONTLIST_DEFAULT_TAG, &choice);
  strcat(base, choice);

    f.value = XmStringCreateLocalized(base);
    ImportColormap(NULL, NULL, &f);

  XtFree(choice);
}


void GoToState(Widget w, XtPointer client, XtPointer call){
    short   which = (int) client;
    short   prevw=0, prevh=0;
    short   k;
    Widget  widget = XtParent(XtParent(daCanvas));

    struct state *temp;
#ifndef SOLARIS
    struct pane  *ptemp;
#endif
    
    switch (which){
        case ABRTCLR:

            XCopyArea(gpDisplay, old, curState->pix, gGC, 0, 0, curState->lWidth, curState->lHeight, 0, 0);
#ifndef SOLARIS
            XCopyArea(gpDisplay, oldpreview, curState->preview, gGC, 0, 0, curState->pwidth, curState->pheight, 0, 0);
            XFreePixmap(gpDisplay, oldpreview);
#endif
            XFreePixmap(gpDisplay, old);
	    return;

            break;

#ifdef ENABLE_JPEG
        case ABRTJPG:
            export_aborted = 1;
            jpeg_destroy((jpeg_common_struct *) &cinfo);
	    return;
            
            break;
#endif

        case ABRTPPM:
            export_aborted = 1;
            fclose(exportFile);
            unlink(export_filename);
	    return;
            
            break;

        case ABRT:
  	    prevw = curState->lWidth;
	    prevh = curState->lHeight;

            temp = curState->prev;
            XFreePixmap(gpDisplay, curState->pix);
#ifndef SOLARIS
            XFreePixmap(gpDisplay, curState->preview);
#endif
            XtFree((char *) curState);
            curState = temp;
            curState->next = firstState;
            lastState = curState;
            firstState->prev = lastState;
#ifndef SOLARIS
            curPane = lastPane;
            ptemp = lastPane->prev;
            XtUnmanageChild(curPane->widget);
            XtDestroyWidget(curPane->widget);
            XtFree((char *) curPane);
            curPane = ptemp;
            curPane->next = firstPane;
            lastPane = curPane;
            firstPane->prev = lastPane;
#endif
            break;
            
        case PREV:
            if (curState==curState->prev) return;

  	    prevw = curState->lWidth;
	    prevh = curState->lHeight;
            curState = curState->prev;
#ifndef SOLARIS
            curPane = curPane->prev;
#endif
            break;

        case NEXT:
            if (curState==curState->next) return;

  	    prevw = curState->lWidth;
	    prevh = curState->lHeight;
            curState = curState->next;
#ifndef SOLARIS
            curPane = curPane->next;
#endif
            break;
    }

    if ((prevh!=curState->lHeight) || (prevw!=curState->lWidth)){
        if (popupmenu)
	    {
            XtUnmanageChild(popupmenu);
            XtDestroyWidget(popupmenu);
	    }

        XtUnmanageChild(daCanvas);
	XtDestroyWidget(daCanvas);
	daCanvas = XtVaCreateManagedWidget("canvas", xmDrawingAreaWidgetClass, widget,
		XmNbackground,       blackpixel,
                XmNwidth,            curState->lWidth,
                XmNheight,           curState->lHeight,
                XmNresizePolicy,     XmRESIZE_ANY,
                NULL);

	create_popupmenu();

	XtAddEventHandler(daCanvas, ExposureMask, false, (XtEventHandler) redraw, NULL);
	XtAddCallback(daCanvas, XmNresizeCallback, (XtCallbackProc) redraw, NULL);
	XtAddEventHandler(daCanvas, ButtonPressMask, false, (XtEventHandler) mouseEvent, (XtPointer) DOWN);
	XtAddEventHandler(daCanvas, ButtonReleaseMask, false, (XtEventHandler) mouseEvent, (XtPointer) UP);
	XtAddEventHandler(daCanvas, ButtonMotionMask, false, (XtEventHandler) mouseEvent, (XtPointer) MOTION);
	XtAddEventHandler(daCanvas, PointerMotionMask, false, (XtEventHandler) mouseEvent, (XtPointer) MOTION);
	XtAddEventHandler(daCanvas, LeaveWindowMask, false, (XtEventHandler) mouseEvent, (XtPointer) OUT);

    }

    for (k=0; k<NUM_COLORS; k++){
        gaDisplayColors[k].red   = curState->colors[k].red;
        gaDisplayColors[k].green = curState->colors[k].green;
        gaDisplayColors[k].blue  = curState->colors[k].blue;

        XAllocColor(gpDisplay, gColormap, &gaDisplayColors[k]);

        if (colorDialog)
            XtVaSetValues(buttons[k], XmNbackground,  gaDisplayColors[k].pixel, NULL);
    }

    if (colorDialog) SelectColormapEntry(NULL, 0, NULL);

    UpdateParametersDialog();

    redraw(daCanvas, NULL, NULL);
#ifndef SOLARIS
    redrawPreviews(NULL, NULL, NULL);
#endif
    gbHasChanged = false;
}







void create_newColormapDialog(Widget w, XtPointer client, XtPointer call){
    /*  new Colormap dialog box  */
    Widget tform, sep, rc, ncform, okbtn;

    if (!newColormapDialog){
        gHiColor.red   = curState->colors[0].red;
        gHiColor.green = curState->colors[0].green;
        gHiColor.blue  = curState->colors[0].blue;
        XAllocColor(gpDisplay, gColormap, &gHiColor);

        gLowColor.red   = curState->colors[NUM_COLORS-1].red;
        gLowColor.green = curState->colors[NUM_COLORS-1].green;
        gLowColor.blue  = curState->colors[NUM_COLORS-1].blue;
        XAllocColor(gpDisplay, gColormap, &gLowColor);

        newColormapDialog = XtVaCreatePopupShell( "New Colormap",
                          xmDialogShellWidgetClass, topLevel,
                          XmNx,    500,       XmNy,     200,
                          XmNwidth,           350,
                          XmNheight,          500,
                          XmNresizePolicy,    XmRESIZE_NONE,
                          XmNdeleteResponse,  XmDESTROY,
                          NULL);

        newColormapPane = XtVaCreateWidget("copane", xmPanedWindowWidgetClass, newColormapDialog,
                          XmNsashWidth,   1,
                          XmNsashHeight,  1,
                          NULL);

        tform  = XtVaCreateManagedWidget("top", xmFormWidgetClass, newColormapPane,
                          XmNnumColumns,          16,
                          XmNorientation,         XmHORIZONTAL,
                          XmNpacking,             XmPACK_TIGHT,
                          XmNleftAttachment,      XmATTACH_FORM,
                          XmNtopAttachment,       XmATTACH_FORM,
                          NULL);

        sep = XtVaCreateManagedWidget("Color 1:", xmLabelWidgetClass, tform,
                          XmNleftAttachment,    XmATTACH_FORM,
                          XmNtopAttachment,     XmATTACH_FORM,
                          XmNheight,            20,
                          NULL);

        rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, tform,
                          XmNorientation,     XmHORIZONTAL,
                          XmNleftAttachment,  XmATTACH_FORM,
                          XmNtopAttachment,   XmATTACH_WIDGET,
                          XmNtopWidget,       sep,
			  NULL);

        prevhi = XtVaCreateManagedWidget("HiPreview", xmDrawingAreaWidgetClass, rc,
                          XmNbackground,   gHiColor.pixel,
                          XmNwidth,        50,
                          XmNheight,       50,
                          NULL);

        rs1 = XtVaCreateManagedWidget("Red", xmScaleWidgetClass, rc,
                          XmNvalue,           gHiColor.red,
                          XmNshowValue,       false,
                          XmNmaximum,         65535,
                          XmNscaleMultiple,   50,
                          XtVaTypedArg,       XmNtitleString,  XmRString, "Red", 4,
                          XtVaTypedArg,       XmNforeground,   XmRString, "Red", 4,
                          NULL);
        XtAddCallback(rs1, XmNdragCallback, UpdateNewColorPreview, (XtPointer) (DoRed * -1));
        XtAddCallback(rs1, XmNvalueChangedCallback, UpdateNewColorPreview, (XtPointer) (DoRed * -1));

        gs1 = XtVaCreateManagedWidget("Green", xmScaleWidgetClass, rc,
                          XmNvalue,           gHiColor.green,
                          XmNshowValue,       false,
                          XmNmaximum,         65535,
                          XmNscaleMultiple,   50,
                          XtVaTypedArg,       XmNtitleString,  XmRString, "Green", 6,
                          XtVaTypedArg,       XmNforeground,   XmRString, "Green", 6,
                          NULL);
        XtAddCallback(gs1, XmNdragCallback, UpdateNewColorPreview, (XtPointer) (DoGreen * -1));
        XtAddCallback(gs1, XmNvalueChangedCallback, UpdateNewColorPreview, (XtPointer) (DoGreen * -1));

        bs1 = XtVaCreateManagedWidget("Blue", xmScaleWidgetClass, rc,
                          XmNvalue,           gHiColor.blue,
                          XmNshowValue,       false,
                          XmNmaximum,         65535,
                          XmNscaleMultiple,   50,
                          XtVaTypedArg,       XmNtitleString,  XmRString, "Blue", 5,
                          XtVaTypedArg,       XmNforeground,   XmRString, "Blue", 5,
                          NULL);
        XtAddCallback(bs1, XmNdragCallback, UpdateNewColorPreview, (XtPointer) (DoBlue * -1));
        XtAddCallback(bs1, XmNvalueChangedCallback, UpdateNewColorPreview, (XtPointer) (DoBlue * -1));

        sep = XtVaCreateManagedWidget("Color 2:", xmLabelWidgetClass, tform,
                          XmNleftAttachment,    XmATTACH_FORM,
                          XmNtopAttachment,     XmATTACH_WIDGET,
                          XmNtopWidget,         rc,
                          XmNtopOffset,         20,
                          XmNheight,            20,
                          NULL);

        rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, tform,
                          XmNorientation,     XmHORIZONTAL,
                          XmNleftAttachment,  XmATTACH_FORM,
                          XmNtopAttachment,   XmATTACH_WIDGET,
                          XmNtopWidget,       sep,
                          NULL);

        prevlow = XtVaCreateManagedWidget("LowPreview", xmDrawingAreaWidgetClass, rc,
                          XmNbackground,   gLowColor.pixel,
                          XmNwidth,        50,
                          XmNheight,       50,
                          NULL);
        rs2 = XtVaCreateManagedWidget("Red", xmScaleWidgetClass, rc,
                          XmNvalue,           gLowColor.red,
                          XmNshowValue,       false,
                          XmNmaximum,         65535,
                          XmNscaleMultiple,   50,
                          XtVaTypedArg,       XmNtitleString,  XmRString, "Red", 4,
                          XtVaTypedArg,       XmNforeground,   XmRString, "Red", 4,
                          NULL);
        XtAddCallback(rs2, XmNdragCallback, UpdateNewColorPreview, (XtPointer) DoRed);
        XtAddCallback(rs2, XmNvalueChangedCallback, UpdateNewColorPreview, (XtPointer) DoRed);

        gs2 = XtVaCreateManagedWidget("Green", xmScaleWidgetClass, rc,
                          XmNvalue,           gLowColor.green,
                          XmNshowValue,       false,
                          XmNmaximum,         65535,
                          XmNscaleMultiple,   50,
                          XtVaTypedArg,       XmNtitleString,  XmRString, "Green", 6,
                          XtVaTypedArg,       XmNforeground,   XmRString, "Green", 6,
                          NULL);
        XtAddCallback(gs2, XmNdragCallback, UpdateNewColorPreview, (XtPointer) DoGreen);
        XtAddCallback(gs2, XmNvalueChangedCallback, UpdateNewColorPreview, (XtPointer) DoGreen);

        bs2 = XtVaCreateManagedWidget("Blue", xmScaleWidgetClass, rc,
                          XmNvalue,           gLowColor.blue,
                          XmNshowValue,       false,
                          XmNmaximum,         65535,
                          XmNscaleMultiple,   50,
                          XtVaTypedArg,       XmNtitleString,  XmRString, "Blue", 5,
                          XtVaTypedArg,       XmNforeground,   XmRString, "Blue", 5,
                          NULL);
        XtAddCallback(bs2, XmNdragCallback, UpdateNewColorPreview, (XtPointer) DoBlue);
        XtAddCallback(bs2, XmNvalueChangedCallback, UpdateNewColorPreview, (XtPointer) DoBlue);


        sep = XtVaCreateManagedWidget("Number of stripes [0-64]:  ", xmLabelWidgetClass, tform,
                          XmNleftAttachment,    XmATTACH_FORM,
                          XmNtopAttachment,     XmATTACH_WIDGET,
                          XmNtopWidget,         rc,
                          XmNheight,            30,
                          NULL);

        numStripes = XtVaCreateManagedWidget("stripesBox", xmTextFieldWidgetClass, tform,
                          XmNvalue,            "2",
                          XmNtopAttachment,    XmATTACH_WIDGET,
                          XmNtopWidget,        rc,
                          XmNleftAttachment,   XmATTACH_WIDGET,
                          XmNleftWidget,       sep,
                          XmNwidth,            50,
                          XmNheight,           30,
		          XmNforeground,       blackpixel,
                          XmNbackground,       whitepixel,
                          NULL);

        ncform = XtVaCreateManagedWidget("action_area", xmFormWidgetClass, newColormapPane,
                          XmNfractionBase,    7,
                          NULL);

        okbtn = XtVaCreateManagedWidget("Generate", xmPushButtonWidgetClass, ncform,
                          XmNleftAttachment,   XmATTACH_POSITION,
                          XmNleftPosition,     1,
                          XmNrightAttachment,  XmATTACH_POSITION,
                          XmNrightPosition,    3,
                          NULL);
        XtAddCallback(okbtn, XmNactivateCallback, GenerateColormap, NULL);

        okbtn = XtVaCreateManagedWidget("Cancel", xmPushButtonWidgetClass, ncform,
                          XmNleftAttachment,   XmATTACH_POSITION,
                          XmNleftPosition,     4,
                          XmNrightAttachment,  XmATTACH_POSITION,
                          XmNrightPosition,    6,
                          NULL);
        XtAddCallback(okbtn, XmNactivateCallback, close_dialog, newColormapDialog);


    }

    XtVaSetValues(rs1, XmNvalue, gHiColor.red, NULL);
    XtVaSetValues(gs1, XmNvalue, gHiColor.green, NULL);
    XtVaSetValues(bs1, XmNvalue, gHiColor.blue, NULL);

    XtVaSetValues(rs2, XmNvalue, gLowColor.red, NULL);
    XtVaSetValues(gs2, XmNvalue, gLowColor.green, NULL);
    XtVaSetValues(bs2, XmNvalue, gLowColor.blue, NULL);
    
    XtVaSetValues(prevhi, XmNbackground, gHiColor.pixel, NULL);
    XtVaSetValues(prevlow, XmNbackground, gLowColor.pixel, NULL);

    XtManageChild(newColormapPane);
    XtManageChild(newColormapDialog);

}


void create_colorDialog(){
    Widget  tform, coform;
    int   i, j=0;
    Widget  okbtn, rc, sep;

  DIR *dp;
  struct dirent *dirp;
  short num = 0;


    if (!colorDialog){

        colorDialog = XtVaCreatePopupShell( "Color Settings",
                   xmDialogShellWidgetClass, topLevel,
                   XmNx,    700,       XmNy,     0,
                   XmNwidth,           350,
                   XmNheight,          500,
                   XmNresizePolicy,    XmRESIZE_NONE,
                   XmNdeleteResponse,  XmDESTROY,
                   NULL);

	copane = XtVaCreateWidget("copane", xmPanedWindowWidgetClass, colorDialog,
                   XmNsashWidth,   1,
                   XmNsashHeight,  1,
                   NULL);
	
        tform  = XtVaCreateManagedWidget("top", xmFormWidgetClass, copane,
                   XmNnumColumns,          16,
                   XmNorientation,         XmHORIZONTAL,
                   XmNpacking,             XmPACK_TIGHT,
                   XmNleftAttachment,      XmATTACH_FORM,
                   XmNbottomAttachment,    XmATTACH_FORM,
                   XmNtopAttachment,       XmATTACH_FORM,
                   NULL);

        for (i=0; i<NUM_COLORS; i++){
            if (i>15) j=1;
            buttons[i] = XtVaCreateManagedWidget(" ", xmDrawingAreaWidgetClass, tform,
                   XmNleftAttachment, (i%16)>0? XmATTACH_WIDGET : XmATTACH_FORM,
                   XmNleftWidget,     (i%16)>0? buttons[i-1] : NULL,
                   XmNtopAttachment,  j>0? XmATTACH_WIDGET : XmATTACH_FORM,
                   XmNtopWidget,      j>0? buttons[i-16] : NULL,
                   XmNbackground,     gaDisplayColors[i].pixel,
                   XmNwidth,          20,
                   XmNheight,         20,
                   NULL);
            XtAddEventHandler(buttons[i], ButtonReleaseMask, false, (XtEventHandler) SelectColormapEntry, (XtPointer) i);
        }

        rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, tform,
                   XmNorientation,     XmHORIZONTAL,
                   XmNleftAttachment,  XmATTACH_FORM,
                   XmNleftOffset,      50,
                   XmNtopAttachment,   XmATTACH_WIDGET,
                   XmNtopWidget,       buttons[NUM_COLORS-1],
                   XmNtopOffset,       10,
                   NULL);

        rscale = XtVaCreateManagedWidget("Red", xmScaleWidgetClass, rc,
                   XmNvalue,           gaDisplayColors[0].red,
                   XmNshowValue,       false,
                   XmNmaximum,         65535,
                   XmNscaleMultiple,   50,
                   XtVaTypedArg,       XmNtitleString,  XmRString, "Red", 4,
                   XtVaTypedArg,       XmNforeground,   XmRString, "Red", 4,
                   NULL);
        XtAddCallback(rscale, XmNdragCallback, UpdateColormapEntryColor, (XtPointer) DoRed);
        XtAddCallback(rscale, XmNvalueChangedCallback, UpdateColormapEntryColor, (XtPointer) DoRed);

        gscale = XtVaCreateManagedWidget("Green", xmScaleWidgetClass, rc,
                   XmNvalue,           gaDisplayColors[0].green,
                   XmNshowValue,       false,
                   XmNmaximum,         65535,
                   XmNscaleMultiple,   50,
                   XtVaTypedArg,       XmNtitleString,  XmRString, "Green", 6,
                   XtVaTypedArg,       XmNforeground,   XmRString, "Green", 6,
                   NULL);
        XtAddCallback(gscale, XmNdragCallback, UpdateColormapEntryColor, (XtPointer) DoGreen);
        XtAddCallback(gscale, XmNvalueChangedCallback, UpdateColormapEntryColor, (XtPointer) DoGreen);

        bscale = XtVaCreateManagedWidget("Blue", xmScaleWidgetClass, rc,
                   XmNvalue,           gaDisplayColors[0].blue,
                   XmNshowValue,       false,
                   XmNmaximum,         65535,
                   XmNscaleMultiple,   50,
                   XtVaTypedArg,       XmNtitleString,  XmRString, "Blue", 5,
                   XtVaTypedArg,       XmNforeground,   XmRString, "Blue", 5,
                   NULL);
        XtAddCallback(bscale, XmNdragCallback, UpdateColormapEntryColor, (XtPointer) DoBlue);
        XtAddCallback(bscale, XmNvalueChangedCallback, UpdateColormapEntryColor, (XtPointer) DoBlue);

        sep = XtVaCreateManagedWidget("rc2", xmRowColumnWidgetClass, rc,
                   XmNorientation,     XmVERTICAL,
                   XmNnumColumns,      1,
                   XmNrightAttachment, XmATTACH_FORM,
                   XmNleftAttachment,  XmATTACH_WIDGET,
                   XmNleftWidget,      bscale,
                   NULL);
        revcolbtn = XtVaCreateManagedWidget("Revert", xmPushButtonWidgetClass, sep,
                   XmNrightAttachment,  XmATTACH_FORM,
                   XmNtopAttachment,    XmATTACH_FORM,
                   XmNsensitive,        false,
                   NULL);
        XtAddCallback(revcolbtn, XmNactivateCallback, RevertColormapEntry, NULL);
        appcolbtn = XtVaCreateManagedWidget("Apply", xmPushButtonWidgetClass, sep,
                   XmNrightAttachment,  XmATTACH_FORM,
                   XmNtopAttachment,    XmATTACH_WIDGET,
                   XmNtopWidget,        revcolbtn,
                   XmNsensitive,        false,
                   NULL);
        XtAddCallback(appcolbtn, XmNactivateCallback, SaveColormapEntry, NULL);
        okbtn = XtVaCreateManagedWidget("Iterate", xmPushButtonWidgetClass, sep,
                   XmNrightAttachment,  XmATTACH_FORM,
                   XmNtopAttachment,    XmATTACH_WIDGET,
                   XmNtopWidget,        appcolbtn,
                   NULL);
        XtAddCallback(okbtn, XmNactivateCallback, IterateButtonPressed, NULL);



	okbtn = XtVaCreateManagedWidget("listform", xmFormWidgetClass, rc,
					NULL);
	sep = XmCreateScrolledList(okbtn, "Maps", NULL, 0);
	XtVaSetValues(sep,
		      XmNwidth, 100,
		      XmNvisibleItemCount, 10,
		      XmNleftAttachment, XmATTACH_FORM,
		      XmNrightAttachment,  XmATTACH_FORM,
		      XmNtopAttachment,    XmATTACH_FORM,
		      XmNbottomAttachment, XmATTACH_FORM,
		      NULL);
  if ( (dp = opendir("./maps")) != NULL )
    {

      while ( (dirp = readdir(dp)) != NULL ){
	num = strlen(dirp->d_name);
	if ((dirp->d_name[num-3]=='m') && (dirp->d_name[num-2]=='a') && (dirp->d_name[num-1]=='p'))
	    XmListAddItem(sep, XmStringCreateLocalized(dirp->d_name), 0);
      }

    }
	XtManageChild(sep);
	XtAddCallback(sep, XmNbrowseSelectionCallback, MapListSelection, NULL);



        coform = XtVaCreateManagedWidget("action_area", xmFormWidgetClass, copane,
                   XmNfractionBase,    13,
                   NULL);

        okbtn = XtVaCreateManagedWidget("Hide", xmPushButtonWidgetClass, coform,
                   XmNleftAttachment,   XmATTACH_POSITION,
                   XmNleftPosition,     1,
                   XmNrightAttachment,  XmATTACH_POSITION,
                   XmNrightPosition,    3,
                   NULL);
        XtAddCallback(okbtn, XmNactivateCallback, close_dialog, colorDialog);

        okbtn = XtVaCreateManagedWidget("New", xmPushButtonWidgetClass, coform,
                   XmNleftAttachment,   XmATTACH_POSITION,
                   XmNleftPosition,     4,
                   XmNrightAttachment,  XmATTACH_POSITION,
                   XmNrightPosition,    6,
                   NULL);
        XtAddCallback(okbtn, XmNactivateCallback, create_newColormapDialog, NULL);

        okbtn = XtVaCreateManagedWidget("Import", xmPushButtonWidgetClass, coform,
                   XmNleftAttachment,   XmATTACH_POSITION,
                   XmNleftPosition,     7,
                   XmNrightAttachment,  XmATTACH_POSITION,
                   XmNrightPosition,    9,
                   NULL);
        XtAddCallback(okbtn, XmNactivateCallback, StateMenuSelect, (XtPointer) LOAD);

        okbtn = XtVaCreateManagedWidget("Export", xmPushButtonWidgetClass, coform,
                   XmNleftAttachment,   XmATTACH_POSITION,
                   XmNleftPosition,     10,
                   XmNrightAttachment,  XmATTACH_POSITION,
                   XmNrightPosition,    12,
                   NULL);
        XtAddCallback(okbtn, XmNactivateCallback, StateMenuSelect, (XtPointer) SAVE);

    }
    else {               
        for (i=0; i<NUM_COLORS; i++) {
            XtVaSetValues(buttons[i], XmNbackground, gaDisplayColors[i].pixel, NULL);
        }
    }

    XtManageChild(copane);
    XtManageChild(colorDialog); 
}


#ifndef SOLARIS
 void create_previewDialog() {//bool override_show) {
    Widget sw, widget;

    if (!previewDialog){

        previewDialog = XtVaCreatePopupShell("Preview Window",
                xmDialogShellWidgetClass, topLevel,
                XmNx,    0,         XmNy,     525,
                XmNwidth,           550,
                XmNheight,          145,
                XmNresizePolicy,    XmRESIZE_NONE,
                XmNdeleteResponse,  XmDESTROY,
                NULL);
        
        pdpane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass, previewDialog,
                XmNsashWidth,   1,
                XmNsashHeight,  1,
                NULL);

        sw = XtVaCreateManagedWidget("sw", xmScrolledWindowWidgetClass, pdpane,
                XmNscrollingPolicy,  XmAUTOMATIC,
                XmNincrement,        10,
                XmNwidth,            550,
                XmNheight,           145,
                NULL);

        previewPane = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, sw,
                XmNorientation,     XmHORIZONTAL,
                XmNleftAttachment,  XmATTACH_FORM,
                XmNtopAttachment,   XmATTACH_FORM,
                XmNspacing,         THUMBNAIL_SPACING,
                NULL);

        widget = XtVaCreateManagedWidget("<-", xmPushButtonGadgetClass, previewPane, NULL);
        XtAddCallback(widget, XmNactivateCallback, GoToState, (XtPointer) PREV);
    
        widget = XtVaCreateManagedWidget("->", xmPushButtonGadgetClass, previewPane, NULL);
        XtAddCallback(widget, XmNactivateCallback, GoToState, (XtPointer) NEXT);

        XtVaSetValues(previewPane, XmNbackground, unselected.pixel, NULL);
    }

    if (!options.bareMode) {//((gbShow) && (override_show)) {
        XtManageChild(pdpane);
        XtManageChild(previewDialog);
        redrawPreviews(NULL, NULL, NULL);
        isPrevShown = true;
    }
}
#endif


short load_and_ExportPPM(char *inputFile){
  /*    FILE    *fd;
    char    buf[32];

    long    w, h, it;
    long    tempx, tempy, n, cc=0;
    short   d, ok=1;
    unsigned short i;
    double  xi, xa, yi, ya, a=0, b=0;
    XColor  co[NUM_COLORS];

    static double  xfa, yfa;
    struct state   temp;
    static struct  complex z;


    //  read file
    if ((fd = fopen(inputFile, "r")) == NULL) {
        printf("File could not be opened\n");
        return -1;
    }

    printf("Reading %s...\n", inputFile);


    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    xi = atof(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    xa = atof(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    yi = atof(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    ya = atof(buf);

    a = b = 0;
    if (giFractalType == JULIA) {
        fscanf(fd, "%s", buf);
        for (i=0; i<strlen(buf); i++){
            if (isalpha(buf[i])){
                ok=0;
                break;
            }
        }
        a = atof(buf);

        fscanf(fd, "%s", buf);
        for (i=0; i<strlen(buf); i++){
            if (isalpha(buf[i])){
                ok=0;
                break;
            }
        }
        b = atof(buf);
    }

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    it = atoi(buf);

    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    w = atoi(buf);


    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    h = atoi(buf);


    fscanf(fd, "%s", buf);
    for (i=0; i<strlen(buf); i++){
        if (isalpha(buf[i])){
            ok=0;
            break;
        }
    }
    d = atoi(buf);


    i=0;
    while (i<NUM_COLORS) {
        fscanf(fd, "%s", buf);
    	co[i].red = atoi(buf);
        fscanf(fd, "%s", buf);
        co[i].green = atoi(buf);
        fscanf(fd, "%s", buf);
        co[i].blue = atoi(buf);

        i++;
    }

    fclose(fd);

    if (ok)
        {
        temp.xmin = xi;
        temp.xmax = xa;
        temp.ymin = yi;
        temp.ymax = ya; 

//        width, height, depth (color/bw)
        temp.lWidth  = w;
        temp.lHeight = h;
        temp.bUseColor  = d;

//        max iterations, julia/mandel
        temp.MaxIterations = it;
        temp.type   = giFractalType;
  
//        which function to iterate
        temp.iFunctionIndex = giFunctionIndex;
  
//        important only if type==JULIA
        temp.parameter.a = a;
        temp.parameter.b = b;

        xfa = (xa-xi)/w;
        yfa = (ya-yi)/h;


        printf("Allocating memory...\n");
        image_buffer = XtMalloc( 3 * w * h * sizeof(int) );


        printf("Generating image values...\n");

        for (tempy=0; tempy<h; tempy++){
            for (tempx=0; tempx<w; tempx++){

                if (giFractalType) {
                    temp.parameter.a = xi + (tempx * xfa);
                    temp.parameter.b = ya - (tempy * yfa);
                    z.a = z.b = 0;
                }
                else {
                    z.a = xi + (tempx * xfa);
                    z.b = ya - (tempy * yfa);
                }
            
                n = iterate_func(it, &z, temp.parameter.a, temp.parameter.b);
                if (n==it)
                    {
                    image_buffer[cc++] = 0;
                    image_buffer[cc++] = 0;
                    image_buffer[cc++] = 0;
                    }
                else
                    {
                    image_buffer[cc++] = co[n % NUM_COLORS].red >> 8;
                    image_buffer[cc++] = co[n % NUM_COLORS].green >> 8;
                    image_buffer[cc++] = co[n % NUM_COLORS].blue >> 8;
                }
            } // x-for
        } //y-for


        printf( "Writing file %s...\n", export_filename );
 
 //     write pixel_map to file
        if ( ( exportFile = fopen( export_filename, "w" ) ) != NULL ) {
            fprintf( exportFile, "P3\n%ld %ld\n%d\n", w, h, 255);
     
            for( i=0; i<3*w*h; i++ )
                fprintf( exportFile, "%ld ", image_buffer[i] );

            fclose(exportFile);
            printf("Export successful.  Exiting...\n\n");
        }
        else
            {
            printf( "fopen returned NULL.\n" );
            return -1;
            }

        } // if (ok)
    else
        {
        printf("Error processing file\n");
        return -1;
        }
  */
    return 0;
}




void signal_handler(int signo){
    /*  signal handler  */
    switch (signo){
        case SIGINT:
            printf("ERROR - Interrupt!\n\n\n");
            byebye(NULL, NULL, NULL);
            break;
        case SIGSEGV:
            printf("ERROR - Seg Fault!\nPlease report all bugs to %s\n\n\n", EMAIL);
            byebye(NULL, NULL, NULL);
            break;
    }
}


void init() {
  options.bareMode = false;
  options.verbose = false;
}




int main(int argc, char **argv){
    Widget   main_w, menubar, widget;
    char     *mapFilename="maps/default.map";
    char     *loadFilename=0;
    short    k;
    XmString load, save, qui, ctrlq, ctrll, st, ctrls, tp, nine, ten;

    XtAppContext app;
    XmFileSelectionBoxCallbackStruct cbs;



    init();


    printf("\nEyeFract %s\nDavid Eyerman\n%s\n\n\n", VERSION, EMAIL);


    for (k=1; k<argc; k++){

        if ((!strcmp(argv[k], "-m")) || (!strcmp(argv[k], "--map"))){
            if (argc>k+1)
                mapFilename = argv[++k];
            else
                printf("Invalid usage:  eyefract -m <map_filename>\nUsing default colormap\n");
        }

        if ((!strcmp(argv[k], "-b")) || (!strcmp(argv[k], "--bare")))
            options.bareMode = true;

        if ((!strcmp(argv[k], "-l")) || (!strcmp(argv[k], "--load"))){
            if (argc>k+1)
                loadFilename = argv[++k];
            else
                {
                printf("Invalid usage:  eyefract -l <state_filename>.  Exiting...\n\n");
                exit(0);
                }
        }


        if ((!strcmp(argv[k], "-o")) || (!strcmp(argv[k], "--output"))){
            if (argc>k+1)
                export_filename = argv[++k];
            else
                {
                printf("Invalid usage:  eyefract -o <target_filename>.  Exiting...\n\n");
                exit(1);
                }
        }

    } //for (reading command line args)

    if (export_filename)
        {
        if (!loadFilename)
            {
            printf("ERROR:  Need input filename (-l option) to use export (-o) from command line\nExiting...\n\n");
            exit(1);
            }

        /*  we have a load filename and an export-to filename  */
        exit( load_and_ExportPPM(loadFilename) );
        }


    /*  so we have an escape  */
#ifdef EYESIGSET
    sigset(SIGINT, signal_handler);
    sigset(SIGSEGV, signal_handler);
#elif defined(EYESIGNAL)
    signal(SIGINT, signal_handler);
    signal(SIGSEGV, signal_handler);
#endif

    topLevel = XtVaAppInitialize(&app, "EyeFract", NULL, 0, &argc, argv, NULL, NULL);

    main_w = XtVaCreateManagedWidget("mw", xmMainWindowWidgetClass, topLevel,
                    XmNwidth,    DEFAULT_WIDTH+50,
                    XmNheight,   DEFAULT_HEIGHT+75,
		    XmNx,        0,
		    XmNy,        0,
                    XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE,
                    NULL);

    /*  menubar  */
    st   = XmStringCreateLocalized("File");
    tp   = XmStringCreateLocalized("State");
    save = XmStringCreateLocalized("View");
    load = XmStringCreateLocalized("Help");
    menubar = XmVaCreateSimpleMenuBar (main_w, "menu",
        XmVaCASCADEBUTTON, st,   'F',
        XmVaCASCADEBUTTON, tp,   'S',
        XmVaCASCADEBUTTON, save, 'V',
        XmVaCASCADEBUTTON, load, 'H',
        NULL);
    XmStringFree(st);
    XmStringFree(load);
    XmStringFree(save);
    XmStringFree(tp);

    /*  create menus  */
    load  = XmStringCreateLocalized("Load state...");
    save  = XmStringCreateLocalized("Save state...");
    st    = XmStringCreateLocalized("Import colormap...");
    qui   = XmStringCreateLocalized("Export colormap...");
#ifdef ENABLE_JPEG
    nine  = XmStringCreateLocalized("Export JPEG...");
#endif
    ten   = XmStringCreateLocalized("Export PPM...");
    tp    = XmStringCreateLocalized("Quit");
    ctrll = XmStringCreateLocalized("Ctrl+L");
    ctrls = XmStringCreateLocalized("Ctrl+S");
    ctrlq = XmStringCreateLocalized("Ctrl+Q");
    XmVaCreateSimplePulldownMenu(menubar, "filemenu", 0, StateMenuSelect,
        XmVaPUSHBUTTON, load, 'L', "Ctrl<Key>L", ctrll,
        XmVaPUSHBUTTON, save, 'S', "Ctrl<Key>S", ctrls,
        XmVaSEPARATOR,
        XmVaPUSHBUTTON, st,   'I', NULL, NULL,
        XmVaPUSHBUTTON, qui,  'E', NULL, NULL,
        XmVaSEPARATOR,
#ifdef ENABLE_JPEG
        XmVaPUSHBUTTON, nine, 'J', NULL, NULL,
#endif
        XmVaPUSHBUTTON, ten,  'P', NULL, NULL,
        XmVaSEPARATOR,
        XmVaPUSHBUTTON, tp,   'Q', "Ctrl<Key>Q", ctrlq,
        NULL);
    XmStringFree(load);
    XmStringFree(save);
#ifdef ENABLE_JPEG
    XmStringFree(nine);
#endif
    XmStringFree(ten);
    XmStringFree(qui);
    XmStringFree(tp);
    XmStringFree(st);
    XmStringFree(ctrll);
    XmStringFree(ctrls);
    XmStringFree(ctrlq);

    /*  state menu - julia or mandelbrot  */
    save  = XmStringCreateLocalized("Julia");
    qui   = XmStringCreateLocalized("Mandelbrot");
    load  = XmStringCreateLocalized("Use Color");
    typeMenu = XmVaCreateSimplePulldownMenu(menubar, "statemenu", 1, TypeMenuSelect,
        XmVaRADIOBUTTON, save, 'J', NULL, NULL,
        XmVaRADIOBUTTON, qui,  'M', NULL, NULL,
        XmVaSEPARATOR,
        XmVaRADIOBUTTON, load, 'C', NULL, NULL,
        NULL);
    XmStringFree(save);
    XmStringFree(load);
    XmStringFree(qui);


    save  = XmStringCreateLocalized("Fractal Parameters");
    qui   = XmStringCreateLocalized("Colormap");
#ifndef SOLARIS
    load  = XmStringCreateLocalized("Preview Pane");
#endif
    XmVaCreateSimplePulldownMenu(menubar, "viewmenu", 2, ViewMenuSelect,
        XmVaPUSHBUTTON, save, 'F', NULL, NULL,
        XmVaPUSHBUTTON, qui,  'C', NULL, NULL,
#ifndef SOLARIS
        XmVaPUSHBUTTON, load, 'P', NULL, NULL,
#endif
        NULL);
    XmStringFree(save);
#ifndef SOLARIS
    XmStringFree(load);
#endif
    XmStringFree(qui);


    load = XmStringCreateLocalized("About EyeFract...");
    XmVaCreateSimplePulldownMenu(menubar, "helpmenu", 3, HelpMenuSelect,
         XmVaPUSHBUTTON, load, 'A', NULL, NULL,
         NULL);
    XmStringFree(load);

#ifdef SOLARIS
    widget = XtVaCreateManagedWidget("   <-   ", xmPushButtonWidgetClass, menubar, NULL);
    XtAddCallback(widget, XmNactivateCallback, GoToState, PREV);
    widget = XtVaCreateManagedWidget("   ->   ", xmPushButtonWidgetClass, menubar, NULL);
    XtAddCallback(widget, XmNactivateCallback, GoToState, NEXT);
#endif
    XtManageChild(menubar);


    /*  the scrolled window  */
    widget = XtVaCreateManagedWidget("sw", xmScrolledWindowWidgetClass, main_w,
                XmNscrollingPolicy,  XmAUTOMATIC,
                XmNincrement,        10,
                NULL);


    sComma = XmStringCreateSimple(", ");
    sOpenPar = XmStringCreateSimple("(");
    sClosePar = XmStringCreateSimple(")");
    nine = XmStringConcat(XmStringCreateSimple("EyeFract "), XmStringCreateSimple(VERSION));
    messageBar = XtVaCreateManagedWidget("messagebar", xmLabelWidgetClass,
        main_w,
        XmNlabelString, nine,
        XmNalignment,   XmALIGNMENT_BEGINNING,
        NULL);
    XmStringFree(nine);

    XmMainWindowSetAreas(main_w, menubar, messageBar, NULL, NULL, widget);
  
    XtRealizeWidget(topLevel);
    gpDisplay    = XtDisplay(topLevel);
    gColormap   = DefaultColormapOfScreen(XtScreen(topLevel));
    gGC = XCreateGC(gpDisplay, XtWindow(topLevel), 0, NULL);
    whitepixel = WhitePixelOfScreen(XtScreen(topLevel));
    blackpixel = BlackPixelOfScreen(XtScreen(topLevel));

#ifndef SOLARIS
    selected.red   = 65535;
    selected.green = 65535;
    selected.blue  = 0;
    unselected.red   = 0;
    unselected.green = 0;
    unselected.blue  = 0;
    XAllocColor(gpDisplay, gColormap, &selected);
    XAllocColor(gpDisplay, gColormap, &unselected);
#endif

    printf("Loading colormap %s...", mapFilename);
    cbs.value = XmStringCreateLocalized(mapFilename);
    ImportColormap(NULL, NULL, &cbs);

    while (!gbLoadColormapOK) {

        if (strcmp(mapFilename, "maps/default.map")==0) {
           printf("ERROR: Default colormap file not found!!\n\n");
           exit(1);
        }

        printf("ERROR: Colormap file %s not found!\nTrying default colormap...", mapFilename);
        strcpy(mapFilename, "maps/default.map");
        cbs.value = XmStringCreateLocalized(mapFilename);
        ImportColormap(NULL, NULL, &cbs);
    }
    printf("\n");
   
    /* where the fractal is to be drawn  */
    daCanvas = XtVaCreateManagedWidget("canvas", xmDrawingAreaWidgetClass, widget,
                XtVaTypedArg, XmNbackground, XmRString, "white", strlen("black"+1),
                XmNwidth,           DEFAULT_WIDTH,
                XmNheight,          DEFAULT_HEIGHT,
                XmNresizePolicy,    XmRESIZE_ANY,
                NULL);
    XtAddEventHandler(daCanvas, ExposureMask, false, (XtEventHandler) redraw, NULL);
    XtAddCallback(daCanvas, XmNresizeCallback, (XtCallbackProc) redraw, NULL);

    if ( (widget = XtNameToWidget(typeMenu, "button_0")) != NULL )
        XtVaSetValues(widget, XmNset, false, NULL);
    if ( (widget = XtNameToWidget(typeMenu, "button_1")) != NULL )
        XtVaSetValues(widget, XmNset, true, NULL);
    XtVaSetValues(XtNameToWidget(typeMenu, "button_2"), XmNset, true, NULL);


    create_paramDialog();

    if (loadFilename)
        {
        printf("Loading state %s...\n", loadFilename);
        cbs.value = XmStringCreateLocalized(loadFilename);

        if (strcmp(mapFilename, "maps/default.map")!=0)
            printf("Warning:  State info will override colormap selection\n");
            
        load_state(NULL, NULL, &cbs);
        }
    else
        {
        NewState(-2.0, 2.0, -2.0, 2.0, DEFAULT_WIDTH, DEFAULT_HEIGHT, true, 100, DEFAULT_TYPE, -.39054, -.58679);
        UpdateParametersDialog();
        CreateFractal(-2.0, 2.0, -2.0, 2.0, CF_NORMAL);
        }


#ifndef SOLARIS
    create_previewDialog();//gbShow);
#endif

    XtAppMainLoop(app);

    return 0;
}
