#include "common.h"
#include "complex.h"
#include "eyemap.h"
#include "xstate.h"
#include "dlgabout.h"
#include "dlgpreview.h"
#include "pane.h"

#include <X11/cursorfont.h>

#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/PanedW.h>
#include <Xm/Scale.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <dirent.h>
#include <signal.h>
#include <unistd.h>
#include <sys/timeb.h>

#ifdef ENABLE_JPEG
#include <jpeglib.h>
#endif




#define LOAD            2
#define SAVE            3
#ifdef ENABLE_JPEG
#define JPEG            4
#endif


/*  for fractal generation  */
#define CF_NORMAL   1
#define CF_COLOR    2
#define CF_PPM      3
#define CF_JPEG     4


/* for mouse events */
#define DOWN        0
#define UP          1
#define MOTION      2
#define OUT         3


/*  whether the coordinates are defined as min/max or center/radius  */
#define CS_MINMAX        0
#define CS_CTRRAD        1



struct {
  bool bareMode;
  bool verbose;
} options;



/*  for passing info when using state context menus  */
struct {
  double xcent, ycent;
} statevars;



double   dZoomFactor = 3.0;
int      giFractalType = DEFAULT_TYPE;       /*  JULIA or MANDELBROT  */


EyeMap   *gpDisplayColors;


bool     gbInit = true;
bool     gbHasChanged = true, stopped;

Widget topLevel, daCanvas;

Widget paramDialog, pane;
Widget funcList;
Widget lblXmin, lblXmax, lblYmin, lblYmax;
Widget xminBox, xmaxBox, yminBox, ymaxBox;
Widget itersBox, zoomBox;
Widget aBox, bBox, widthBox, heightBox;
Widget keepaspect, coordStyleBtn;
short  giCoordDisplayStyle = CS_MINMAX;
long   giOldWidth=-1, giOldHeight=-1;


Widget colorDialog;
Widget buttons[NUM_COLORS];
Widget copane, appcolbtn, revcolbtn;
Widget rscale, bscale, gscale;

Widget popupmenu;
Widget typeMenu, messageBar, ccpane;
Widget viewMenu;

Widget waitDialog;

dlgAbout   *aboutDialog;
dlgPreview *previewDialog = 0;


Widget newColormapDialog, newColormapPane, prevhi, prevlow, numStripes;
Widget rs1, rs2, gs1, gs2, bs1, bs2;

Colormap  gColormap;
Display   *gpDisplay;
GC        gGC;
Window    gWindow;


Pixel     whitepixel, blackpixel;


short     curCol = -1;//, prevCol = -1;        /*  index of current, previously selected colors in colors[]  */
XColor    gHiColor, gLowColor;              /*  used for creating colormaps  */
XColor    oldColor;


XmString  sComma, sOpenPar, sClosePar;

Pixmap    old;

FILE      *exportFile;
short     export_aborted=0;
char      *export_filename;


/*  for exporting JPEGs  */
#ifdef ENABLE_JPEG
JSAMPLE   *image_buffer;	/* Points to large array of R,G,B-order data */
struct    jpeg_compress_struct cinfo;
#else
long      *image_buffer;
#endif

XState *firstState, *lastState, *curState;


void create_colorDialog(bool show);
void create_paramDialog(bool show);
void create_previewDialog(bool isInit, bool hide);

void CreateFractal(short);
void DeleteState(Widget w, XtPointer a, XtPointer b);
void GoToState(XState *s);
void NewState(double xn, double xm, double yn, double ym, long w, long h, bool useColor, long i,
	      short type, double a, double b);
void ParameterEdited(Widget w, XtPointer client, XtPointer callback);
void PopupMenuSelect(Widget widget, XtPointer client, XtPointer call);
void RefreshParameters();
void SelectColormapEntry(Widget, XtPointer, XtPointer);
