#include "eyemap.h"

class MapException {
public:
  MapException(char *message) : message (message) { }
  MapException() : message ("my message") { }
  const char *what() const { return message; }

private:
  const char *message;
};



EyeMap::EyeMap(Display *dpy, Colormap *cmap, XColor *cols) {
  short i;

  gpDisplay = dpy;
  gColormap = (*cmap);

  for (i=0; i<NUM_COLORS; i++)
    {
      SetEntry(i, (*cols));
      cols++;
    }
}



EyeMap::EyeMap(char *filename, Display *dpy, Colormap *cmap) {
  /*  import colormap from a file  */
  FILE    *fd;
  char    t[12];
  short   k;

  try {

    /*  read file  */
    if ((fd = fopen(filename, "r")) == NULL) {
      printf("ERROR: Colormap %s could not be opened\n", filename);
      return;
    }

    k=0;
    while (k < NUM_COLORS) {
        fscanf(fd, "%s", t);
	if (isValueOK(t, "red"))
	  colors[k].red = atoi(t);
	else
	  throw MapException("Red");

        fscanf(fd, "%s", t);
	if (isValueOK(t, "green"))
	  colors[k].green = atoi(t);
	else
	  throw MapException("Green");

        fscanf(fd, "%s", t);
	if (isValueOK(t, "blue"))
	  colors[k].blue = atoi(t);
	else
	  throw MapException("Blue");

        k++;
    }
    fclose(fd);

    if (dpy)
      {
      gpDisplay = dpy;
      gColormap = (*cmap);

      for (k=0; k<NUM_COLORS; k++)
	XAllocColor(gpDisplay, gColormap, &colors[k]);
      }
  }
  catch (MapException me) {
    printf("ERROR: %s is not a valid colormap file\n", filename);
    fclose(fd);
  }

}


EyeMap::EyeMap(short type, char *filename, Display *dpy, Colormap *cmap, bool *bUseColor) {
  char t[20];
  FILE *fd;
  short i;

  try {

  fd = fopen(filename, "r");

  if (type==JULIA)
    for (i=0; i<11; i++)
      fscanf (fd, "%s", t);
  else
    for (i=0; i<9; i++)
      fscanf(fd, "%s", t);

  (*bUseColor) = (t[0]=='1');

  i=0;
  while (i<NUM_COLORS) {
    fscanf(fd, "%s", t);
    if (isValueOK(t, "red"))
      colors[i].red = atoi(t);
    else
      throw MapException("Red");

    
    fscanf(fd, "%s", t);
    if (isValueOK(t, "green"))
      colors[i].green = atoi(t);
    else
      throw MapException("Green");

    fscanf(fd, "%s", t);
    if (isValueOK(t, "blue"))
      colors[i].blue = atoi(t);
    else
      throw MapException("Blue");

    i++;
  }

  fclose(fd);

  if (dpy)
    {
    gpDisplay = dpy;
    gColormap = (*cmap);
    for (i=0; i<NUM_COLORS; i++)
      XAllocColor(gpDisplay, gColormap, &colors[i]);

    if (i!=NUM_COLORS)
      throw MapException("Not enough numbers");
    }
  }
  catch (MapException me) {
    printf("ERROR: %s does not contain valid colormap info\n", filename);
    fclose(fd);
  }

}



void EyeMap::SetEntry(short index, XColor value) {
  colors[index].red   = value.red;
  colors[index].green = value.green;
  colors[index].blue  = value.blue;

  XAllocColor(gpDisplay, gColormap, &colors[index]);
}


EyeMap::~EyeMap() {

}




void EyeMap::StoreColor(short index, XColor *saveme){
  saveme->red   = colors[index].red;
  saveme->green = colors[index].green;
  saveme->blue  = colors[index].blue;

  XAllocColor(gpDisplay, gColormap, saveme);
}



void EyeMap::UpdateEntry(short index, short whichComponent, long value){
  switch (whichComponent) {
    case DoRed:
      colors[index].red   = value;
      break;
    case DoGreen:
      colors[index].green = value;
      break;
    case DoBlue:
      colors[index].blue  = value;
      break;
  }

  XAllocColor(gpDisplay, gColormap, &colors[index]);

}



int EyeMap::WriteToFile(char *filename) {
    /*  export colormap to a file  */
    FILE   *fd;
    short  i;
    
    /*  write file  */
    if ((fd = fopen(filename, "w")) == NULL) {
        printf("ERROR: %s could not be opened for writing\n", filename);
        return -1;
    }

    for (i=0; i<NUM_COLORS; i++)
        fprintf(fd, "%d %d %d\n", colors[i].red, colors[i].green, colors[i].blue);

    fclose(fd);

    return 0;
}
