#include "function.h"


Function::Function() { }

Function::~Function() { }



Function *GetNewFunction(short id, double a, double b) {
  Function *retval = 0;

  switch (id) {
  case 0:
    retval = new Quadratic(a, b);
    break;

  case 1:
    return new Quadratic2(a, b);
    break;

  case 2:
    return new Quartic(a, b);
    break;

  case 3:
    return new Quartic2(a, b);
    break;

  case 4:
    return new Quintic(a, b);
    break;
    
  case 5:
    return new PiIEz();
    break;

  case 6:
    return new SineZ();
    break;

  case 7:
    return new cCosineZ(a, b);
    break;
  }

  return retval;
}



//
//  f(z) = z^2 + c
//
/////////////////////////////////////
Quadratic::Quadratic() {
  parameter.a = parameter.b = 0;
  Init();
}

Quadratic::Quadratic(double a, double b){
  parameter.a = a;
  parameter.b = b;
  Init();
}

Quadratic::~Quadratic() {
}

void Quadratic::Init() {
  label = "f(z) = z^2 + c";
  iFunctionType  = BOTH;
  iFunctionIndex = 0;
}

long Quadratic::Iterate(EComplex *z) {
  static long n;
  static EComplex t;

  n = 0;

  while ((z->GetSqLength()<4) && (n<lMaxIterations)) {
    t.a = z->a;
    t.b = z->b;

    z->a = (t.a * t.a) - (t.b * t.b) + parameter.a;
    z->b = (2 * t.a * t.b) + parameter.b;
    
    n++;
  }

  return n;
}



//
//  f(z) = z^2 + cz
//
////////////////////////////////////////////////
Quadratic2::Quadratic2() {
  parameter.a = parameter.b = 0;
  Init();
}

Quadratic2::Quadratic2(double a, double b){
  parameter.a = a;
  parameter.b = b;
  Init();
}

Quadratic2::~Quadratic2() {
}

void Quadratic2::Init() {
  label = "f(z) = z^2 + c*z (Julia only)";
  iFunctionType  = JULIA_ONLY;
  iFunctionIndex = 1;
}

long Quadratic2::Iterate(EComplex *z) {
  long n = 0;
  EComplex t;

  t.a = z->a;
  t.b = z->b;
  while ((z->GetSqLength()<4) && (n<lMaxIterations)) {
    z->a = (t.a * parameter.a) + (t.a * t.a) - (t.b * parameter.b) - (t.b * t.b);
    z->b = (t.a * parameter.b) + (t.b * parameter.a) + (2 * t.a * t.b);

    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
}




//
//  f(z) = z^4 + c
//
/////////////////////////////////////////////
Quartic::Quartic(){
  parameter.a = parameter.b = 0;
  Init();
}

Quartic::Quartic(double a, double b){
  parameter.a = a;
  parameter.b = b;
  Init();
}

Quartic::~Quartic() {
}

void Quartic::Init() {
  label = "f(z) = z^4 + c";
  iFunctionType  = BOTH;
  iFunctionIndex = 2;
}

long Quartic::Iterate(EComplex *z) {
  EComplex t;
  long n = 0;

  t.a = z->a;
  t.b = z->b;

  while ((z->GetSqLength()<4) && (n<lMaxIterations)) {
    z->a = (t.a * t.a * t.a * t.a) - (6 * t.a * t.a * t.b * t.b) + (t.b * t.b * t.b * t.b) + parameter.a;
    z->b = 4 * t.a * t.b * (t.a * t.a - t.b * t.b) + parameter.b;
    
    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
}




//
//  f(z) = z^4 - z + c
//
//////////////////////////////////
Quartic2::Quartic2(){
  parameter.a = parameter.b = 0;
  Init();
}

Quartic2::Quartic2(double a, double b){
  parameter.a = a;
  parameter.b = b;
  Init();
}

Quartic2::~Quartic2() {
}

void Quartic2::Init() {
  label = "f(z) = z^4 - z + c";
  iFunctionType  = BOTH;
  iFunctionIndex = 3;
}

long Quartic2::Iterate(EComplex *z) {
  EComplex t;
  long n = 0;

  t.a = z->a;
  t.b = z->b;

  while ((z->GetSqLength()<4) && (n<lMaxIterations)) {
    z->a = (t.a * t.a * t.a * t.a) - (6 * t.a * t.a * t.b * t.b) + (t.b * t.b * t.b * t.b) - t.a + parameter.a;
    z->b = 4 * t.a * t.b * (t.a * t.a - t.b * t.b) - t.b + parameter.b;

    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
}





//
//  f(z) = z^5 + c
//
////////////////////////////////////////////////
Quintic::Quintic(){
  parameter.a = parameter.b = 0;
  Init();
}

Quintic::Quintic(double a, double b){
  parameter.a = a;
  parameter.b = b;
  Init();
}

Quintic::~Quintic() {
}

void Quintic::Init() {
  label = "f(z) = z^5 + c";
  iFunctionType  = BOTH;
  iFunctionIndex = 4;
}

long Quintic::Iterate(EComplex *z) {
  EComplex t;
  long n = 0;

  t.a = z->a;
  t.b = z->b;

  while ((z->GetSqLength()<4) && (n<lMaxIterations)) {
    z->a = (t.a * t.a * t.a * t.a * t.a) - (10 * t.a * t.a * t.a * t.b * t.b) + (5 * t.a * t.b * t.b * t.b * t.b) + parameter.a;
    z->b = (t.b * t.b * t.b * t.b * t.b) - (10 * t.a * t.a * t.b * t.b * t.b) + (5 * t.a * t.a * t.a * t.a * t.b) + parameter.b;

    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
}




//
//  f(z) = pi * i * e^z
//
//////////////////////////////////////////////////
PiIEz::PiIEz(){
  parameter.a = parameter.b = 0;
  Init();
}

PiIEz::~PiIEz() {
}

void PiIEz::Init(){
  label = "f(z) = pi * i * e^z";
  iFunctionType  = BOTH;
  iFunctionIndex = 5;
}

long PiIEz::Iterate(EComplex *z) {
  EComplex t;
  long n = 0;

  t.a = z->a;
  t.b = z->b;

  while ((t.a<10) && (n<lMaxIterations)) {
    z->a = -1 * M_PI * exp(t.a) * cos(t.b);
    z->b = M_PI * exp(t.a) * sin(t.b);

    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
}




//
//  f(z) = sin z
//
/////////////////////////////////////////
SineZ::SineZ(){
  Init();
}

SineZ::~SineZ(){
}

void SineZ::Init(){
  label = "f(z) = sin ( z )";
  iFunctionType  = BOTH;
  iFunctionIndex = 6;
}

long SineZ::Iterate(EComplex *z){
  EComplex t;
  long n = 0;


  while ((t.a<50) && (n<lMaxIterations)) {
    z->a = sin(t.a)*(exp(t.b)+exp(-1*t.b))/2;
    z->b = cos(t.a)*(exp(t.b)-exp(-1*t.b))/2;
    
    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
  }




//
//  f(z) = a * cos z
//
////////////////////////////////////////////////////
cCosineZ::cCosineZ(){
  parameter.a = parameter.b = 0;
  Init();
}

cCosineZ::cCosineZ(double a, double b){
  parameter.a = a;
  parameter.b = b;
  Init();
}

cCosineZ::~cCosineZ(){
}

void cCosineZ::Init(){
  label = "f(z) = c * cos( z )";
  iFunctionType  = BOTH;
  iFunctionIndex = 7;
}

long cCosineZ::Iterate(EComplex *z){
  long n=0;
  EComplex t;
  t.a = z->a;
  t.b = z->b;


  while ((t.a<50) && (n<lMaxIterations)) {
    z->a = (cos(t.a)*(exp(t.b)+exp(-1*t.b))*parameter.a - sin(t.a)*(exp(-1*t.b)-exp(t.b))*parameter.b)/2;
    z->b = (sin(t.a)*(exp(-1*t.b)-exp(t.b))*parameter.a + cos(t.a)*(exp(-1*t.b)-exp(t.b))*parameter.b)/2;
    
    t.a = z->a;
    t.b = z->b;

    n++;
  }

  return n;
}



