#ifndef FUNCTION_H
#define FUNCTION_H

#include "common.h"
#include "complex.h"
#include <math.h>

#define JULIA_ONLY 1
#define BOTH       2


class Function {
 public:

  short          iFunctionType;
  char          *label;
  long           lMaxIterations;
  short          iFunctionIndex;
  EComplex parameter;

  Function();
  virtual ~Function();

  virtual long Iterate(EComplex *z) = 0;

};


class Quadratic : public Function {
 public:

  //static const char * const label = "f(z) = z^2 + c";

  Quadratic(double a, double b);
  Quadratic();
  ~Quadratic();

  void Init();
  long Iterate(EComplex *z);
};


class Quadratic2 : public Function {
 public:
  //    static const char * const label = "f(z) = z^2 + c*z (Julia only)";

  Quadratic2(double a, double b);
  Quadratic2();
  ~Quadratic2();

  void Init();
  long Iterate(EComplex *z);
};


class Quartic : public Function {
 public:
  //    static const char * const label = "f(z) = z^4 + c";

  Quartic(double a, double b);
  Quartic();
  ~Quartic();

  void Init();
  long Iterate(EComplex *z);
};


class Quartic2 : public Function {
 public:
  //    static const char * const label = "f(z) = z^4 - z +  c";

  Quartic2(double a, double b);
  Quartic2();
  ~Quartic2();

  void Init();
  long Iterate(EComplex *z);
};


class Quintic : public Function {
 public:
  //    static const char * const label = "f(z) = z^5 + c";

  Quintic(double a, double b);
  Quintic();
  ~Quintic();

  void Init();
  long Iterate(EComplex *z);
};


class PiIEz : public Function {
 public:
  //    static const char * const label = "f(z) = pi * i * e^z";

  PiIEz();
  ~PiIEz();

  void Init();
  long Iterate(EComplex *z);
};


class SineZ : public Function {
 public:
  //    static const char * const label = "f(z) = sin(z)";

  SineZ();
  ~SineZ();

  void Init();
  long Iterate(EComplex *z);
};


class cCosineZ : public Function {
 public:
  //  static const char * const label = "f(z) = c * cos(z)";

  cCosineZ();
  cCosineZ(double a, double b);
  ~cCosineZ();

  void Init();
  long Iterate(EComplex *z);
};


Function *GetNewFunction(short id, double a, double b);

//STILL TO DO:
    /* pi * i * tan z */
    /* (1 + .1*i)sin z */
    /* e^(z-1) */
    /* (1/e + .1) * e^z */
    /* (.6 + .8i) * sin z */

const char * const function_lbls[] = { "f(z) = z^2 + c",   "f(z) = z^2 + c*z (Julia only)",
				       "f(z) = z^4 + c",   "f(z) = z^4 - z + c",
				       "f(z) = z^5 + c",   "f(z) = pi * i * e^z",
				       "f(z) = sin ( z )", "f(z) = c * cos( z )" };

const short giNumFuncs = 8;



#endif
