#include "pane.h"


void Redraw(Widget w, XtPointer client, XEvent *event){
  Pane *tempPane = (Pane *) client;
  long neww, newh;

  //  printf("id=%d\tdpy=%d\timage=%d\n", tempPane->id, tempPane->gpDisplay, tempPane->image);
  //printf("win=%d\n", XtWindow(tempPane->widget));
  //printf("gc=%d\tw=%d\th=%d\n", tempPane->gGC, tempPane->image_width, tempPane->image_height);

  neww = THUMBNAIL_PADDING/2+((THUMBNAIL_SIZE-(tempPane->image_width))/2);
  newh = THUMBNAIL_PADDING/2+((THUMBNAIL_SIZE-(tempPane->image_height))/2);

  XCopyArea(tempPane->gpDisplay, tempPane->image, XtWindow(w), tempPane->gGC, 0, 0, tempPane->image_width, tempPane->image_height, neww, newh);

}


void ThumbnailSelect(Widget widget, XtPointer client, XEvent *event){

  GoToState((int) client);

}





Pane::Pane(Widget parent, bool isFirst, short newid, GC *gc, Pane *first, Pane *last, long w, long h) {

  id  = newid;

  gpScreen  = 0;
  gpDisplay = 0;
  gWindow   = 0;
  gGC       = 0;
  widget    = 0;
  image     = 0;

  if (first!=0)
    {
      prev = last;
      next = first;
    }
  else
    {
      prev = this;
      next = prev;
    }

  if (w >= h) {
    image_width  = 100;
    image_height = 100*h/w;
  } else {
    image_width  = 100*w/h;
    image_height = 100;
  }

  widthRatio  = (double) image_width  / w;
  heightRatio = (double) image_height / h;

  if (parent)
    SetDisplay(parent, gc, isFirst);

}




Pane::~Pane() {

  if (gWindow)
    {
      XtUnmanageChild(widget);
      XtDestroyWidget(widget);
    }

}



void Pane::GeneratePreview(Pixmap source){
  short tempx, tempy;
  long sendw = 0, sendh = 0;

  for (tempx=0; tempx<image_width; tempx++){
    for (tempy=0; tempy<image_height; tempy++){
      sendw = (long) floor(tempx / widthRatio);
      sendh = (long) floor(tempy / heightRatio);
      XCopyArea(gpDisplay, source, image, gGC, sendw, sendh, 1, 1, tempx, tempy);
    }
  }
}





void Pane::Refresh() {
  Redraw(widget, (XtPointer) this, NULL);
}





void Pane::SetDisplay(Widget parent, GC *gc, bool isFirst){
  gpDisplay = XtDisplay(parent);
  gpScreen  = XtScreen(parent);
  gGC       = (*gc);

  widget =  XtVaCreateManagedWidget("sep", xmDrawingAreaWidgetClass, parent,
				    XmNbackground,       BlackPixelOfScreen(gpScreen),
				    XmNwidth,            THUMBNAIL_SIZE+THUMBNAIL_PADDING,
				    XmNheight,           THUMBNAIL_SIZE+THUMBNAIL_PADDING,
				    XmNresizePolicy,     XmRESIZE_ANY,
				    XmNleftAttachment,   (isFirst)? XmATTACH_WIDGET : XmATTACH_FORM,
				    XmNleftWidget,       (isFirst)? prev->widget    : NULL,
				    NULL);
  gWindow   = XtWindow(widget);

  image = XCreatePixmap(gpDisplay, gWindow, image_width, image_height, DefaultDepthOfScreen(gpScreen));
  XFillRectangle(gpDisplay, image, gGC, 0, 0, image_width, image_height);

  XtAddCallback(widget, XmNresizeCallback, (XtCallbackProc) Redraw, (XtPointer) this);
  XtAddEventHandler(widget, ExposureMask,      false, (XtEventHandler) Redraw,          (XtPointer) this);
  XtAddEventHandler(widget, ButtonReleaseMask, false, (XtEventHandler) ThumbnailSelect, (XtPointer) id);
}

