#include "state.h"

long State::giHighestID = 0;

class MyException {
public:
  MyException(char *message) : message (message) { }
  MyException() : message ("my message") { }
  const char *what() const { return message; }

private:
  const char *message;
};



bool isValueOK(char *value)
{
  unsigned int i;

  for (i=0; i<strlen(value); i++){
    if ( isalpha(value[i]) ) 
      return false;
  }

  return true;
}




State::State(double xn, double xm, double yn, double ym, long w, long h, long i,
                short typ, Function *f){

  id = GetID();

  dXmin = xn;
  dXmax = xm;
  dYmin = yn;
  dYmax = ym; 

  lWidth  = w;
  lHeight = h;

  lMaxIterations = i;
  type   = typ;
    
  function = f;

}



State::State(char *filename) {

  char          buf[32];
  FILE          *fd;

    //
    //
    //  HERE!!! ERROR CHECKING!!!!
    //
    //
    ////////////////////////////////

  id = GetID();

  /*  read file  */
  fd = fopen(filename, "r");


  if (filename[strlen(filename)-3]=='j')
    type = JULIA;
  else
    type = MANDELBROT;

  try {

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      dXmin = atof(buf);
    else
      throw MyException("Xmin");

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      dXmax = atof(buf);
    else
      throw MyException("Xmax");

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      dYmin = atof(buf);
    else
      throw MyException("Ymin");

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      dYmax = atof(buf);
    else
      throw MyException("Ymax");

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      {
	function = GetNewFunction(atoi(buf), 0, 0);
	giFunctionIndex = atoi(buf);
      }
    else
      throw MyException("Function Index");

    Init();

    function->parameter.a = function->parameter.b = 0;
    if (type == JULIA) {

        fscanf(fd, "%s", buf);
	if ( isValueOK(buf) )
	  function->parameter.a = atof(buf);
	else
	  throw MyException("a");

        fscanf(fd, "%s", buf);
	if ( isValueOK(buf) )
	  function->parameter.b = atof(buf);
	else
	  throw MyException("b");
    }

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      lMaxIterations = atoi(buf);
    else
      throw MyException("num its");

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      lWidth = atoi(buf);
    else
      throw MyException("width");

    fscanf(fd, "%s", buf);
    if ( isValueOK(buf) )
      lHeight = atoi(buf);
    else
      throw MyException("height");


    fclose(fd);

    }
    catch (MyException myex){
      printf("ERROR: %s is not a valid state file\n", filename);
      fclose(fd);
    }


}





State::~State() {
  if (function)
    delete function;
}




void State::Dump(){
  printf("\nSTATE DUMP\nID: %ld\nXmin: %.15f\t Xmax: %.15f\nYmin: %.15f\tYmax:%.15f\nWidth=%ld\tHeight=%ld\nIters:%ld\n",
	 id, dXmin, dXmax, dYmin, dYmax, lWidth, lHeight, lMaxIterations);
}



long State::GetID()
{
  return ++giHighestID;
}



void State::Init()
{
  function = GetNewFunction(giFunctionIndex, 0, 0);
}



int State::WriteToFile(char *filename) {
  FILE   *fd;
    
  if ((fd = fopen(filename, "w")) == NULL) {
    printf("ERROR: %s could not be opened for writing\n", filename);
    return -1;
  }

  fprintf(fd, "%.15f %.15f %.15f %.15f", dXmin, dXmax, dYmin, dYmax);

  fprintf(fd, " %d", function->iFunctionIndex);

  if (type == JULIA)
    fprintf(fd, " %.15f %.15f", function->parameter.a, function->parameter.b);

  fprintf(fd, " %ld %ld %ld\n", lMaxIterations, lWidth, lHeight);

  fclose(fd);

  return 0;
}
